#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from pyecharts import Polar

def test_polar():

    # polar_0
    import random
    data = [(i, random.randint(1, 100)) for i in range(101)]
    polar = Polar("极坐标系-散点图示例")
    polar.add("", data, boundary_gap=False, type='scatter', is_splitline_show=False, is_axisline_show=True)
    polar.show_config()
    polar.render()

    # polar_1
    data_1 = [(10, random.randint(1, 100)) for i in range(300)]
    data_2 = [(11, random.randint(1, 100)) for i in range(300)]
    polar = Polar("极坐标系-散点图示例", width=1200, height=600)
    polar.add("", data_1, type='scatter')
    polar.add("", data_2, type='scatter')
    polar.show_config()
    polar.render()

    # porlar_2
    data = [(i, random.randint(1, 100)) for i in range(10)]
    polar = Polar("极坐标系-动态散点图示例", width=1200, height=600)
    polar.add("", data, type='effectScatter', effect_scale=10, effect_period=5)
    polar.show_config()
    polar.render()

    # polar_3
    radius = ['周一', '周二', '周三', '周四', '周五', '周六', '周日']
    polar = Polar("极坐标系-堆叠柱状图示例", width=1200, height=600)
    polar.add("A", [1, 2, 3, 4, 3, 5, 1], radius_data=radius, type='barRadius', is_stack=True)
    polar.add("B", [2, 4, 6, 1, 2, 3, 1], radius_data=radius, type='barRadius', is_stack=True)
    polar.add("C", [1, 2, 3, 4, 1, 2, 5], radius_data=radius, type='barRadius', is_stack=True)
    polar.show_config()
    polar.render()

    # polar_4
    radius = ['周一', '周二', '周三', '周四', '周五', '周六', '周日']
    polar = Polar("极坐标系-堆叠柱状图示例", width=1200, height=600)
    polar.add("", [1, 2, 3, 4, 3, 5, 1], radius_data=radius, type='barAngle', is_stack=True)
    polar.add("", [2, 4, 6, 1, 2, 3, 1], radius_data=radius, type='barAngle', is_stack=True)
    polar.add("", [1, 2, 3, 4, 1, 2, 5], radius_data=radius, type='barAngle', is_stack=True)
    polar.show_config()
    polar.render()

    # polar_5
    import math
    data = []
    for i in range(101):
        theta = i / 100 * 360
        r = 5 * (1 + math.sin(theta / 180 * math.pi))
        data.append([r, theta])
    hour = [i for i in range(1, 25)]
    polar = Polar("极坐标系示例", width=1200, height=600)
    polar.add("Love", data, angle_data=hour, boundary_gap=False, start_angle=0)
    polar.show_config()
    polar.render()

    # polar_6
    data = []
    for i in range(361):
        t = i / 180 * math.pi
        r = math.sin(2 * t) * math.cos(2 * t)
        data.append([r, i])
    polar = Polar("极坐标系示例", width=1200, height=600)
    polar.add("Flower", data, start_angle=0, symbol=None, axis_range=[0, None])
    polar.show_config()
    polar.render()

    # polar_7
    data = []
    for i in range(361):
        t = i / 180 * math.pi
        r = math.sin(2 * t) * math.cos(2 * t)
        data.append([r, i])
    polar = Polar("极坐标系示例", width=1200, height=600)
    polar.add("Color-Flower", data, start_angle=0, symbol=None, axis_range=[0, None],
              area_color="#f71f24", area_opacity=0.6)
    polar.show_config()
    polar.render()

    # polar_8
    data = []
    for i in range(5):
        for j in range(101):
            theta = j / 100 * 360
            alpha = i * 360 + theta
            r = math.pow(math.e, 0.003 * alpha)
            data.append([r, theta])
    polar = Polar("极坐标系示例")
    polar.add("", data, symbol_size=0, symbol='circle', start_angle=-25, is_radiusaxis_show=False,
              area_color="#f3c5b3", area_opacity=0.5, is_angleaxis_show=False)
    polar.show_config()
    polar.render()



#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base
from pyecharts.option import get_all_options

class Pie(Base):
    """
    <<< Pie chart >>>
    The pie chart is mainly used for showing proportion of different categories.
    Each arc length represents the proportion of data quantity.
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(Pie, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, attr, value,
              radius=None,
              center=None,
              rosetype=None,
              **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param attr:
            name of attribute
        :param value:
            value of attribute
        :param radius:
            Radius of Pie chart, the first of which is inner radius, and the second is outer radius.
            Percentage is supported. When set in percentage,
            it's relative to the smaller size between height and width of the container.
        :param center:
            Center position of Pie chart, the first of which is the horizontal position,
            and the second is the vertical position.
            Percentage is supported. When set in percentage, the item is relative to the container width,
            and the second item to the height.
        :param rosetype:
            Whether to show as Nightingale chart, which distinguishs data through radius. There are 2 optional modes:
            'radius' Use central angle to show the percentage of data, radius to show data size.
            'area' All the sectors will share the same central angle, the data size is shown only through radiuses.
        :param kwargs:
        """
        kwargs.update(type="pie")
        chart = get_all_options(**kwargs)
        assert len(attr) == len(value)
        _data = []
        for data in zip(attr, value):
            _name, _value = data
            _data.append({"name": _name, "value": _value})

        _rmin, _rmax = "0%", "75%"
        if radius:
            if len(radius) == 2:
                _rmin, _rmax = ["{}%".format(r) for r in radius]

        _cmin, _cmax = "50%", "50%"
        if center:
            if len(center) == 2:
                _cmin, _cmax = ["{}%".format(c) for c in center]

        if rosetype:
            if rosetype not in ("radius", "area"):
                rosetype = "radius"
        for a in attr:
            self._option.get('legend')[0].get('data').append(a)
        _dlst = self._option.get('legend')[0].get('data')
        _dset = list(set(_dlst))
        _dset.sort(key=_dlst.index)
        self._option.get('legend')[0].update(data=list(_dset))
        self._option.get('series').append({
            "type": "pie",
            "name": name,
            "data": _data,
            "radius": [_rmin, _rmax],
            "center": [_cmin, _cmax],
            "roseType": rosetype,
            "label": chart['label'],
            "indexflag": self._option.get('_index_flag')
        })
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from pyecharts import Pie

def test_pie():

    # pie_0
    attr = ["衬衫", "羊毛衫", "雪纺衫", "裤子", "高跟鞋", "袜子"]
    v1 = [11, 12, 13, 10, 10, 10]
    pie = Pie("饼图示例")
    pie.add("", attr, v1, is_label_show=True)
    pie.show_config()
    pie.render()

    # pie_1
    attr = ["衬衫", "羊毛衫", "雪纺衫", "裤子", "高跟鞋", "袜子"]
    v1 = [11, 12, 13, 10, 10, 10]
    pie = Pie("饼图-圆环图示例", title_pos='center')
    pie.add("", attr, v1, radius=[40, 75], label_text_color=None, is_label_show=True, legend_orient='vertical',
            legend_pos='left')
    pie.show_config()
    pie.render()

    # pie_2
    attr = ["衬衫", "羊毛衫", "雪纺衫", "裤子", "高跟鞋", "袜子"]
    v1 = [11, 12, 13, 10, 10, 10]
    v2 = [19, 21, 32, 20, 20, 33]
    pie = Pie("饼图-玫瑰图示例", title_pos='center', width=900)
    pie.add("商品A", attr, v1, center=[25, 50], is_random=True, radius=[30, 75], rosetype='radius')
    pie.add("商品B", attr, v2, center=[75, 50], is_random=True, radius=[30, 75], rosetype='area', is_legend_show=False,
            is_label_show=True)
    pie.show_config()
    pie.render()

    # pie_3
    pie = Pie("饼图示例", title_pos='center', width=1000, height=600)
    pie.add("", ['A', 'B', 'C', 'D', 'E', 'F'], [335, 321, 234, 135, 251, 148], radius=[40, 55], is_label_show=True)
    pie.add("", ['H', 'I', 'J'], [335, 679, 204], radius=[0, 30], legend_orient='vertical', legend_pos='left')
    pie.show_config()
    pie.render()

    # pie_4
    import random
    attr = ['A', 'B', 'C', 'D', 'E', 'F']
    pie = Pie("饼图示例", width=1000, height=600)
    pie.add("", attr, [random.randint(0, 100) for _ in range(6)], radius=[50, 55], center=[25, 50], is_random=True)
    pie.add("", attr, [random.randint(20, 100) for _ in range(6)], radius=[0, 45], center=[25, 50], rosetype='area')
    pie.add("", attr, [random.randint(0, 100) for _ in range(6)], radius=[50, 55], center=[65, 50], is_random=True)
    pie.add("", attr, [random.randint(20, 100) for _ in range(6)], radius=[0, 45], center=[65, 50], rosetype='radius')
    pie.show_config()
    pie.render()

    # Pie_5
    pie = Pie('各类电影中"好片"所占的比例', "数据来着豆瓣", title_pos='center')
    pie.add("", ["剧情", ""], [25, 75], center=[10, 30], radius=[18, 24],
            label_pos='center', is_label_show=True, label_text_color=None, )
    pie.add("", ["奇幻", ""], [24, 76], center=[30, 30], radius=[18, 24],
            label_pos='center', is_label_show=True, label_text_color=None, legend_pos='left')
    pie.add("", ["爱情", ""], [14, 86], center=[50, 30], radius=[18, 24],
            label_pos='center', is_label_show=True, label_text_color=None)
    pie.add("", ["惊悚", ""], [11, 89], center=[70, 30], radius=[18, 24],
            label_pos='center', is_label_show=True, label_text_color=None)
    pie.add("", ["冒险", ""], [27, 73], center=[90, 30], radius=[18, 24],
            label_pos='center', is_label_show=True, label_text_color=None)
    pie.add("", ["动作", ""], [15, 85], center=[10, 70], radius=[18, 24],
            label_pos='center', is_label_show=True, label_text_color=None)
    pie.add("", ["喜剧", ""], [54, 46], center=[30, 70], radius=[18, 24],
            label_pos='center', is_label_show=True, label_text_color=None)
    pie.add("", ["科幻", ""], [26, 74], center=[50, 70], radius=[18, 24],
            label_pos='center', is_label_show=True, label_text_color=None)
    pie.add("", ["悬疑", ""], [25, 75], center=[70, 70], radius=[18, 24],
            label_pos='center', is_label_show=True, label_text_color=None)
    pie.add("", ["犯罪", ""], [28, 72], center=[90, 70], radius=[18, 24],
            label_pos='center', is_label_show=True, label_text_color=None, is_legend_show=True, legend_top="center")
    pie.show_config()
    pie.render()



#coding=utf-8
from __future__ import unicode_literals

from nose.tools import eq_
from pyecharts.template import freeze_js
from pyecharts.template import write_utf8_html_file
import codecs


def test_freeze_js():
    html_content = """
        </style>
        <!-- build -->
        <script src="js/echarts/echarts.min.js"></script>
        <script src="js/echarts/echarts-wordcloud.min.js"></script>
        <!-- endbuild -->
    </head><body>"""

    html_content = freeze_js(html_content)
    assert 'exports.echarts' in html_content
    assert 'echarts-wordcloud' in html_content


def test_write_utf8_html_file():
    content = "柱状图数据堆叠示例"
    file_name = 'test.html'
    write_utf8_html_file(file_name, content)
    with codecs.open(file_name, 'r', 'utf-8') as f:
        actual_content = f.read()
        eq_(content, actual_content)



#!/usr/bin/env python
#coding=utf-8

from pyecharts import WordCloud

def test_wordcloud():

    # wordcloud_0
    name = ['Sam S Club', 'Macys', 'Amy Schumer', 'Jurassic World', 'Charter Communications',
            'Chick Fil A', 'Planet Fitness', 'Pitch Perfect', 'Express', 'Home', 'Johnny Depp',
            'Lena Dunham', 'Lewis Hamilton', 'KXAN', 'Mary Ellen Mark', 'Farrah Abraham',
            'Rita Ora', 'Serena Williams', 'NCAA baseball tournament', 'Point Break']
    value = [10000, 6181, 4386, 4055, 2467, 2244, 1898, 1484, 1112, 965, 847, 582, 555,
             550, 462, 366, 360, 282, 273, 265]

    wordcloud = WordCloud(width=1300, height=620)
    wordcloud.add("", name, value, word_size_range=[30, 100], rotate_step=66)
    wordcloud.show_config()
    wordcloud.render()

    # wordcloud_1
    wordcloud = WordCloud(width=1300, height=620)
    wordcloud.add("", name, value, word_size_range=[30, 100], shape='diamond')
    wordcloud.show_config()
    # wordcloud.render()



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from pyecharts import HeatMap

def test_heatmap():

    # heatmap_0
    import random
    x_axis = ["12a", "1a", "2a", "3a", "4a", "5a", "6a", "7a", "8a", "9a", "10a", "11a",
              "12p", "1p", "2p", "3p", "4p", "5p", "6p", "7p", "8p", "9p", "10p", "11p"]
    y_aixs = ["Saturday", "Friday", "Thursday", "Wednesday", "Tuesday", "Monday", "Sunday"]
    data = [[i, j, random.randint(0, 50)] for i in range(24) for j in range(7)]
    heatmap = HeatMap("热力图示例")
    heatmap.add("热力图直角坐标系", x_axis, y_aixs, data, is_visualmap=True,
                visual_text_color="#000", visual_orient='horizontal')
    heatmap.show_config()
    heatmap.render()



import pprint

class Timeline(object):

    def __init__(self, is_auto_play=False,
                 is_loop_play=True,
                 is_rewind_play=False,
                 is_timeline_show=True,
                 timeline_play_interval=2000,
                 timeline_symbol="emptyCircle",
                 timeline_symbol_size=10,
                 timeline_left="auto",
                 timeline_right="auto",
                 timeline_top="auto",
                 timeline_bottom="atuo"):
        """

        :param is_auto_play:
            Whether to play automatically.
        :param is_loop_play:
            Whether to loop playing.
        :param is_rewind_play:
            Whether supports playing reversely.
        :param is_timeline_show:
            Whether to show the timeline component.
            It would not show with a setting of false, but its functions still remain.
        :param timeline_play_interval:
            Indicates play speed (gap time between two state), whose unit is millisecond.
        :param timeline_symbol:
            Symbol of timeline.
            Icon types provided by ECharts includes 'circle', 'rect', 'roundRect', 'triangle', 'diamond', 'pin', 'arrow'
        :param timeline_symbol_size:
            timeline symbol size.
            It can be set to single numbers like 10, or use an array to represent width and height.
            For example, [20, 10] means symbol width is 20, and height is10.
        :param timeline_left:
            Distance between timeline component and the left side of the container.
        :param timeline_right:
            Distance between timeline component and the right side of the container.
        :param timeline_top:
            Distance between timeline component and the top side of the container.
        :param timeline_bottom:
            Distance between timeline component and the bottom side of the container.
        """

        self._chart = None
        self._time_points = []
        self._timeline_options = {
            "baseOption": {
                "timeline": {
                    "axisType": "category",
                    "autoPlay": is_auto_play,
                    "loop": is_loop_play,
                    "rewind": is_rewind_play,
                    "show": is_timeline_show,
                    "symbol": timeline_symbol,
                    "symbolSize": timeline_symbol_size,
                    "playInterval": timeline_play_interval,
                    "left": timeline_left,
                    "right": timeline_right,
                    "top": timeline_top,
                    "bottom": timeline_bottom
            }},
            "options": []
        }

    def add(self, chart, time_point):
        """

        :param chart:
        :param time_point:
        :return:
        """
        self._chart = chart
        self.__check_components(chart)
        self._time_points.append(time_point)
        self._timeline_options.get('baseOption').update(
            legend=chart._option.get('legend'),
            backgroundColor=chart._option.get('backgroundColor')
        )
        self._timeline_options.get('baseOption').get('timeline').update(
            data=self._time_points
        )
        self._timeline_options.get('options').append({
            "series": chart._option.get('series'),
            "title": chart._option.get('title')
        })

    def show_config(self):
        """

        :return:
        """
        pprint.pprint(self._timeline_options)

    def render(self, path="render.html"):
        """

        :param path:
        :return:
        """
        self._chart._option = self._timeline_options
        self._chart.render(path)

    def render_embed(self):
        """

        :return:
        """
        self._chart._option = self._timeline_options
        return self._chart.render_embed()

    @property
    def chart(self):
        """

        :return:
        """
        return self._chart

    def _repr_html_(self):
        """

        :return:
        """
        return self._chart._repr_html_()

    def __check_components(self, chart):
        """

        :param chart:
        :return:
        """
        _grid = chart._option.get('grid', None)
        if _grid is not None:
            self._timeline_options.get('baseOption').update(grid=_grid)

        _xaxis = chart._option.get('xAxis', None)
        if _xaxis is not None:
            self._timeline_options.get('baseOption').update(xAxis=_xaxis)

        _yaxis = chart._option.get('yAxis', None)
        if _yaxis is not None:
            self._timeline_options.get('baseOption').update(yAxis=_yaxis)

        _polar = chart._option.get('polar', None)
        if _polar is not None:
            self._timeline_options.get('baseOption').update(polar=_polar)

        _radiusAxis = chart._option.get('radiusAxis', None)
        if _radiusAxis is not None:
            self._timeline_options.get('baseOption').update(radiusAxis=_radiusAxis)

        _angleAxis = chart._option.get('angleAxis', None)
        if _angleAxis is not None:
            self._timeline_options.get('baseOption').update(angleAxis=_angleAxis)

        _radar = chart._option.get('radar', None)
        if _radar is not None:
            self._timeline_options.get('baseOption').update(radar=_radar)

        _visualMap = chart._option.get('visualMap', None)
        if _visualMap is not None:
            self._timeline_options.get('baseOption').update(visualMap=_visualMap)

        _geo = chart._option.get('geo', None)
        if _geo is not None:
            self._timeline_options.get('baseOption').update(geo=_geo)

        _datazoom = chart._option.get('dataZoom', None)
        if _geo is not None:
            self._timeline_options.get('baseOption').update(dataZoom=_datazoom)

        _parallelAxis = chart._option.get('parallelAxis', None)
        if _parallelAxis is not None:
            self._timeline_options.get('baseOption').update(parallelAxis=_parallelAxis)



#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base

class Liquid(Base):
    """
    <<< Liquid chart >>>
    Liquid chart is usually used to represent data in percentage.
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(Liquid, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, data,
              shape='circle',
              liquid_color=None,
              is_liquid_animation=True,
              is_liquid_outline_show=True):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param data:
            data of liquid,
            [0.6, 0.5, 0.4, 0.3] -> This creates a chart wit waves at position of 60%, 50%, 40%, and 30%.
        :param shape:
            Shape of water fill chart.
            It can be one of the default symbols: 'circle', 'rect', 'roundRect', 'triangle', 'diamond', 'pin', 'arrow'
        :param liquid_color:
            To set colors for liquid fill chart series, set color to be an array of colors.
        :param is_liquid_animation:
            Whether disable animation.
        :param is_liquid_outline_show:
            whether hide the outline
        :return:
        """
        _animation_dur, _animation_dur_update = 2000, 1000
        if not is_liquid_animation:
            _animation_dur, _animation_dur_update = 0, 0

        _color = ['#294D99', '#156ACF', '#1598ED', '#45BDFF']
        if liquid_color:
            _color = liquid_color

        _shape = 'circle'
        if shape in ('circle', 'rect', 'roundRect', 'triangle', 'diamond', 'pin', 'arrow'):
            _shape = shape

        self._option.get('series').append({
            "type": "liquidFill",
            "name": name,
            "data": data,
            "waveAnimation": is_liquid_animation,
            "animationDuration": _animation_dur,
            "animationDurationUpdate": _animation_dur_update,
            "color": _color,
            "shape": _shape,
            "outline": {"show": is_liquid_outline_show}
        })



#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base
from pyecharts.option import gen_color

class WordCloud(Base):
    """
    <<< WordCloud chart >>>
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(WordCloud, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, attr, value,
              shape="circle",
              word_gap=20,
              word_size_range=None,
              rotate_step=45):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param attr:
            name of attribute
        :param value:
            value of attribute
        :param shape:
            shape of wordcloud
            It can be 'circle', 'cardioid', 'diamond', 'triangle-forward', 'triangle', 'pentagon', 'star'
        :param word_gap:
            Gap of word
            size of the grid in pixels for marking the availability of the canvas
            the larger the grid size, the bigger the gap between words.
        :param word_size_range:
            Text size range which the value in data will be mapped to.
            Default to have minimum 12px and maximum 60px size.
        :param rotate_step:
            Text rotation range and step in degree. Text will be rotated randomly in range [-90, 90] by rotationStep 45
        """
        assert len(attr) == len(value)
        _data = []
        for data in zip(attr, value):
            _name, _value = data
            _data.append({
                "name": _name,
                "value": _value,
                "textStyle": {
                    "normal": {"color": gen_color()}}
            })
        _min, _max = 12, 60
        if word_size_range is not None:
            if len(word_size_range) == 2:
                _min, _max = word_size_range

        _rmin, _rmax = -90, 90
        # make sure shape valid, need to set ratated range [-90, 90]
        if shape in ("cardioid", "diamond", "triangle-forward", "triangle", "pentagon", "star"):
            _rmin = _rmax = 0
        else:
            shape = "circle"
        self._option.get('series').append({
            "type": "wordCloud",
            "name": name,
            "shape": shape,
            "rotationRange": [_rmin, _rmax],
            "rotationStep": rotate_step,
            "girdSize": word_gap,
            "sizeRange": [_min, _max],
            "data": _data
        })



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from pyecharts import Line3D

def test_line3D():

    # line3D_0
    import math
    _data = []
    for t in range(0, 25000):
        _t = t / 1000
        x = (1 + 0.25 * math.cos(75 * _t)) * math.cos(_t)
        y = (1 + 0.25 * math.cos(75 * _t)) * math.sin(_t)
        z = _t + 2.0 * math.sin(75 * _t)
        _data.append([x, y, z])
    range_color = ['#313695', '#4575b4', '#74add1', '#abd9e9', '#e0f3f8', '#ffffbf',
                   '#fee090', '#fdae61', '#f46d43', '#d73027', '#a50026']
    line3d = Line3D("3D 折线图示例", width=1200, height=600)
    line3d.add("", _data, is_visualmap=True, visual_range_color=range_color, visual_range=[0, 30],
               grid3D_rotate_sensitivity=5)
    line3d.render()

    # line3D_1
    import math
    _data = []
    for t in range(0, 25000):
        _t = t / 1000
        x = (1 + 0.25 * math.cos(75 * _t)) * math.cos(_t)
        y = (1 + 0.25 * math.cos(75 * _t)) * math.sin(_t)
        z = _t + 2.0 * math.sin(75 * _t)
        _data.append([x, y, z])
    range_color = ['#313695', '#4575b4', '#74add1', '#abd9e9', '#e0f3f8', '#ffffbf',
                   '#fee090', '#fdae61', '#f46d43', '#d73027', '#a50026']
    line3d = Line3D("3D 折线图示例", width=1200, height=600)
    line3d.add("", _data, is_visualmap=True, visual_range_color=range_color, visual_range=[0, 30],
               is_grid3D_rotate=True, grid3D_rotate_speed=180)
    line3d.render()




#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base
from pyecharts.option import get_all_options

class Line(Base):
    """
    <<< Broken Line >>>
    Broken line chart relates all the data points symbol by broken lines,
    which is used to show the trend of data changing.
    It could be used in both rectangular coordinate and polar coordinate.
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(Line, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, x_axis, y_axis,
              is_symbol_show=True,
              is_smooth=False,
              is_stack=False,
              is_step=False,
              is_fill=False,
              **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param x_axis:
            data of xAxis
        :param y_axis:
            data of yAxis
        :param is_symbol_show:
            It specifies whether to show the symbol.
        :param is_smooth:
            Whether to show as smooth curve.
        :param is_stack:
            It specifies whether to stack category axis.
        :param is_step:
            Whether to show as a step line. It can be true, false. Or 'start', 'middle', 'end'.
            Which will configure the turn point of step line.
        :param is_fill:
            Whether to fill area.
        :param kwargs:
        """
        assert len(x_axis) == len(y_axis)
        kwargs.update(x_axis=x_axis, type="line")
        chart = get_all_options(**kwargs)
        xaxis, yaxis = chart['xy_axis']
        is_stack = "stack" if is_stack else ""
        _area_style = {"normal": chart['area_style']} if is_fill else {}
        self._option.update(xAxis=xaxis, yAxis=yaxis)
        self._option.get('legend')[0].get('data').append(name)
        self._option.get('series').append({
            "type": "line",
            "name": name,
            "symbol": chart['symbol'],
            "smooth": is_smooth,
            "step": is_step,
            "stack": is_stack,
            "showSymbol": is_symbol_show,
            "data": y_axis,
            "label": chart['label'],
            "lineStyle": chart['line_style'],
            "areaStyle": _area_style,
            "markPoint": chart['mark_point'],
            "markLine": chart['mark_line'],
            "indexflag": self._option.get('_index_flag')
        })
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from pyecharts import Scatter

def test_scatter():

    # scatter_0
    v1 = [10, 20, 30, 40, 50, 60]
    v2 = [10, 20, 30, 40, 50, 60]
    scatter = Scatter("散点图示例")
    scatter.add("A", v1, v2)
    scatter.add("B", v1[::-1], v2)
    scatter.show_config()
    scatter.render()

    # scatter_0_1
    scatter = Scatter("散点图示例")
    scatter.add("A", v1, v2)
    scatter.add("B", v1[::-1], v2, is_visualmap=True)
    scatter.show_config()
    scatter.render()

    # scatter_0_2
    scatter = Scatter("散点图示例")
    scatter.add("A", v1, v2)
    scatter.add("B", v1[::-1], v2, is_visualmap=True, visual_type='size', visual_range_size=[20, 80])
    scatter.show_config()
    scatter.render()

    # scatter_0_3
    scatter = Scatter("散点图示例")
    scatter.add("A", ["a", "b", "c", "d", "e", "f"], v2)
    scatter.add("B", ["a", "b", "c", "d", "e", "f"], v1[::-1], xaxis_type="category")
    scatter.show_config()
    scatter.render()

    # scatter_1
    scatter = Scatter("散点图示例")
    v1, v2 = scatter.draw("../images/pyecharts-0.png")
    scatter.add("pyecharts", v1, v2, is_random=True)
    scatter.show_config()
    scatter.render()

    # scatter_2
    scatter = Scatter("散点图示例", width=800, height=480)
    v1, v2 = scatter.draw("../images/love.png")
    scatter.add("Love", v1, v2)
    scatter.render()

    # scatter_3
    scatter = Scatter("散点图示例", width=1000, height=480)
    v1, v2 = scatter.draw("../images/cup.png")
    scatter.add("Cup", v1, v2)
    scatter.render()

    # scatter_4
    scatter = Scatter("散点图示例", width=1000, height=480)
    v1, v2 = scatter.draw("../images/cup.png")
    scatter.add("Cup", v1, v2, label_color=["#000"])
    scatter.render()



#!/usr/bin/env python
#coding=utf-8

from pyecharts._version import __version__, __author__

# charts
from pyecharts.charts.bar import Bar
from pyecharts.charts.bar3D import Bar3D
from pyecharts.charts.effectscatter import EffectScatter
from pyecharts.charts.funnel import Funnel
from pyecharts.charts.gauge import Gauge
from pyecharts.charts.geo import Geo
from pyecharts.charts.graph import Graph
from pyecharts.charts.heatmap import HeatMap
from pyecharts.charts.kline import Kline
from pyecharts.charts.line import Line
from pyecharts.charts.line3D import Line3D
from pyecharts.charts.liquid import Liquid
from pyecharts.charts.map import Map
from pyecharts.charts.parallel import Parallel
from pyecharts.charts.pie import Pie
from pyecharts.charts.polar import Polar
from pyecharts.charts.radar import Radar
from pyecharts.charts.scatter import Scatter
from pyecharts.charts.scatter3D import Scatter3D
from pyecharts.charts.wordcloud import WordCloud

# custom component
from pyecharts.custom.grid import Grid
from pyecharts.custom.overlap import Overlap
from pyecharts.custom.page import Page
from pyecharts.custom.timeline import Timeline

# version
from pyecharts._version import __version__


#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base

class Parallel(Base):
    """
    <<< Parallel chart >>>
    Parallel Coordinates is a common way of visualizing high-dimensional geometry and analyzing multivariate data.
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(Parallel, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def config(self, schema=None, c_schema=None):
        """

        :param schema:
            Dimension index of coordinate axis.
            a axis name list, like ['apple', 'orange', 'watermelon']
        :param c_schema:
            User customize coordinate axis for parallel coordinate.
            dim: Dimension index of coordinate axis.
            name: Name of axis.
            type: Type of axis
                value: Numerical axis, suitable for continuous data.
                category: Category axis, suitable for discrete category data.
                          Data should only be set via data for this type.
            min: The minimun value of axis.
            max: The maximum value of axis.
            inverse: Whether axis is inversed.
            nameLocation: Location of axis name. it can be 'start', 'middle', 'end'.
        :return:
        """
        if schema:
            _schema = [{"dim": i, "name": v} for i, v in enumerate(schema)]
            self._option.update(parallelAxis=_schema)
        if c_schema:
            self._option.update(parallelAxis=c_schema)

    def __add(self, name, data, **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param data:
            data array of series, it is represented by a two-dimension array -> [[],[]]
        :param kwargs:
        :return:
        """
        self._option.update(parallel={"left": "5%", "right": "13%", "bottom": "10%", "top": "20%"})
        self._option.get('legend')[0].get('data').append(name)
        self._option.get('series').append({
            "type": "parallel",
            "coordinateSystem": "parallel",
            "name": name,
            "data": data,
        })
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
#coding=utf-8

from PIL import Image

from pyecharts.base import Base
from pyecharts.option import get_all_options


class Scatter(Base):
    """
    <<< Scatter chart >>>
    The scatter chart in rectangular coordinate could be used to present the relation between x and y.
    If data have multiple dimensions, the values of the other dimensions can be visualized through symbol with
    various sizes and colors, which becomes a bubble chart. These can be done by using with visualMap component.
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(Scatter, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, x_axis, y_axis,
              symbol_size=10,
              **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param x_axis:
            data of xAxis
        :param y_axis:
            data of yAxis
        :param symbol_size:
            symbol size
        :param kwargs:
        """
        assert len(x_axis) == len(y_axis)
        kwargs.update(type="scatter", x_axis=x_axis)
        chart = get_all_options(**kwargs)
        xaxis, yaxis = chart['xy_axis']
        self._option.update(xAxis=xaxis, yAxis=yaxis)
        self._option.get('legend')[0].get('data').append(name)
        self._option.get('series').append({
            "type": "scatter",
            "name": name,
            "symbol": chart['symbol'],
            "symbolSize": symbol_size,
            "data": [list(z) for z in zip(x_axis, y_axis)],
            "label": chart['label'],
            "indexflag": self._option.get('_index_flag')
        })
        self._legend_visualmap_colorlst(**kwargs)

    def draw(self, path, color=None):
        """ Converts the pixels on the image to an array

        :param path:
            path of Image that want to draw
        :param color:
            select a color to exclude, (225, 225, 225) means Keep only white pixel information.
        :return:
        """
        color = color or (255, 255, 255)
        im = Image.open(path)
        width, height = im.size
        imarray = im.load()
        # flip vertical Images
        for x in range(width):
            for y in range(height):
                if y < int(height / 2):
                    imarray[x, y], imarray[x, height-y-1] = imarray[x, height-y-1], imarray[x, y]
        # [:3] is r,g,b
        result = [(x, y) for x in range(width) for y in range(height) if imarray[x, y][:3] != color]
        return self.cast(result)



#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base
from pyecharts.option import get_all_options

class Line3D(Base):
    """
    <<< Line3D chart >>>
    """

    def __init__(self, title="", subtitle="", **kwargs):
        super(Line3D, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, data,
              grid3D_opacity=1,
              **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param data:
            data of line3D
        :param grid3D_opacity:
            opacity of gird3D item
        :param kwargs:
        :return:
        """
        kwargs.update(xaxis3d_type='value', yaxis3d_type='value', zaxis3d_type='value')
        chart = get_all_options(**kwargs)
        self._option.get('legend')[0].get('data').append(name)
        self._option.update(
            xAxis3D=chart['xaxis3D'],
            yAxis3D=chart['yaxis3D'],
            zAxis3D=chart['zaxis3D'],
            grid3D=chart['grid3D']
        )
        self._option.get('series').append({
            "type": "line3D",
            "name": name,
            "data": data,
            "label": chart['label'],
            "itemStyle": {"opacity": grid3D_opacity}
        })
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from pyecharts import Gauge

def test_gague():

    # gauge_0
    gauge = Gauge("仪表盘示例")
    gauge.add("业务指标", "完成率", 66.66)
    gauge.show_config()
    gauge.render()

    # gauge_1
    gauge = Gauge("仪表盘示例")
    gauge.add("业务指标", "完成率", 166.66, angle_range=[180, 0], scale_range=[0, 200], is_legend_show=False)
    gauge.show_config()
    gauge.render()



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from pyecharts import Funnel

def test_funnel():

    # funnel_0
    attr = ["衬衫", "羊毛衫", "雪纺衫", "裤子", "高跟鞋", "袜子"]
    value = [20, 40, 60, 80, 100, 120]
    funnel = Funnel("漏斗图示例")
    funnel.add("商品", attr, value, is_label_show=True, label_pos="inside", label_text_color="#fff")
    funnel.render()

    # funnel_1
    funnel = Funnel("漏斗图示例", width=600, height=400, title_pos='center')
    funnel.add("商品", attr, value, is_label_show=True, label_pos="outside", legend_orient='vertical', legend_pos='left')
    funnel.show_config()
    funnel.render()



#!/usr/bin/env python
# coding=utf-8

from pyecharts.option import grid

class Grid(object):

    def __init__(self):
        self._chart = None

    def add(self, chart,
            grid_width=None,
            grid_height=None,
            grid_top=None,
            grid_bottom=None,
            grid_left=None,
            grid_right=None):
        """

        :param chart:
            chart instance
        :param grid_width:
            Width of grid component. Adaptive by default.
        :param grid_height:
            Height of grid component. Adaptive by default.
        :param grid_top:
            Distance between grid component and the top side of the container.
        :param grid_bottom:
            Distance between grid component and the bottom side of the container.
        :param grid_left:
            Distance between grid component and the left side of the container.
        :param grid_right:
            Distance between grid component and the right side of the container.
        :return:
        """
        if self._chart is None:
            self._chart = chart
            self._chart._option.update(grid=[])

            _grid = grid(grid_width, grid_height, grid_top, grid_bottom, grid_left, grid_right)
            if _grid:
                for _ in range(len(self._chart._option.get('series'))):
                    self._chart._option.get('grid').append(_grid)
        else:
            _index, _index_once, _xaxis, _yaxis, _legned, _title = self.__custom(self.__get_series(chart))
            self._chart._option.get('legend').append(_legned)
            self._chart._option.get('title').append(_title)
            if _xaxis and _yaxis is not None:
                try:
                    _xaxis[0].update(gridIndex=_index - 1)
                    _yaxis[0].update(gridIndex=_index - 1)
                    self._chart._option.get('xAxis').append(_xaxis[0])
                    self._chart._option.get('yAxis').append(_yaxis[0])
                except:
                    pass

                # indexflag is only identify for every series
                _flag = self._chart._option.get('series')[0].get('indexflag')
                _series_index = 0
                for s in self._chart._option.get('series'):
                    if _flag == s.get('indexflag'):
                        s.update(xAxisIndex=_series_index, yAxisIndex=_series_index)
                    else:
                        _series_index += 1
                        s.update(xAxisIndex=_series_index, yAxisIndex=_series_index)
                    _flag = s.get('indexflag')
            _grid = grid(grid_width, grid_height, grid_top, grid_bottom, grid_left, grid_right)
            for _ in range(_index_once):
                self._chart._option.get('grid').append(_grid)

    def __get_series(self, chart):
        """ Get chart series data """
        return (
            chart._option.get('legend')[0].get('data'),
            chart._option.get('series'),
            chart._option.get('xAxis', None),
            chart._option.get('yAxis', None),
            chart._option.get('legend')[0],
            chart._option.get('title')[0]
        )

    def __custom(self, series):
        """

        :param series:
            series data
        :return:
        """
        _name, _series, _xaxis, _yaxis, _legend, _title = series
        for s in _series:
            self._chart._option.get('series').append(s)
        return len(self._chart._option.get('series')), len(_series), _xaxis, _yaxis, _legend, _title

    def render(self, path="render.html"):
        """

        :param path:
        :return:
        """
        self._chart.render(path)

    def render_embed(self):
        """

        :return:
        """
        return self._chart.render_embed()

    def show_config(self):
        """

        :return:
        """
        import pprint
        return pprint.pprint(self._chart._option)

    @property
    def chart(self):
        """

        :return:
        """
        return self._chart

    def _repr_html_(self):
        """

        :return:
        """
        return self._chart._repr_html_()



#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base
from pyecharts.option import get_all_options

class Kline(Base):
    """
    <<< Kline(Candlestick) chart >>>
    Kline chart use red to imply increasing with red and decreasing with blue
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(Kline, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, x_axis, y_axis, **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param x_axis:
            data of xAxis
        :param y_axis:
            data pf yAxis
            Data should be the two-dimensional array shown as follow. -> [[],[]]
            Every data item (each line in the example above) represents a box, which contains 4 values. They are:
            [open, close, lowest, highest]  (namely: [opening value, closing value, lowest value, highest value])
        :param kwargs:
        :return:
        """
        kwargs.update(type="candlestick", x_axis=x_axis)
        chart = get_all_options(**kwargs)
        xaxis, yaxis = chart['xy_axis']
        self._option.update(xAxis=xaxis, yAxis=yaxis)
        self._option.get('legend')[0].get('data').append(name)
        self._option.get('series').append({
            "type": "candlestick",
            "name": name,
            "data": y_axis,
            "markPoint": chart['mark_point'],
            "markLine": chart['mark_line'],
            "indexflag": self._option.get('_index_flag')
        })
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from random import randint

from pyecharts import Bar, Pie, Line
from pyecharts.custom.timeline import Timeline


def test_timeline():

    attr = ["衬衫", "羊毛衫", "雪纺衫", "裤子", "高跟鞋", "袜子"]
    bar_1 = Bar("2012 年销量", "数据纯属虚构")
    bar_1.add("春季", attr, [randint(10, 100) for _ in range(6)])
    bar_1.add("夏季", attr, [randint(10, 100) for _ in range(6)])
    bar_1.add("秋季", attr, [randint(10, 100) for _ in range(6)])
    bar_1.add("冬季", attr, [randint(10, 100) for _ in range(6)])

    bar_2 = Bar("2013 年销量", "数据纯属虚构")
    bar_2.add("春季", attr, [randint(10, 100) for _ in range(6)])
    bar_2.add("夏季", attr, [randint(10, 100) for _ in range(6)])
    bar_2.add("秋季", attr, [randint(10, 100) for _ in range(6)])
    bar_2.add("冬季", attr, [randint(10, 100) for _ in range(6)])

    bar_3 = Bar("2014 年销量", "数据纯属虚构")
    bar_3.add("春季", attr, [randint(10, 100) for _ in range(6)])
    bar_3.add("夏季", attr, [randint(10, 100) for _ in range(6)])
    bar_3.add("秋季", attr, [randint(10, 100) for _ in range(6)])
    bar_3.add("冬季", attr, [randint(10, 100) for _ in range(6)])

    bar_4 = Bar("2015 年销量", "数据纯属虚构")
    bar_4.add("春季", attr, [randint(10, 100) for _ in range(6)])
    bar_4.add("夏季", attr, [randint(10, 100) for _ in range(6)])
    bar_4.add("秋季", attr, [randint(10, 100) for _ in range(6)])
    bar_4.add("冬季", attr, [randint(10, 100) for _ in range(6)])

    bar_5 = Bar("2016 年销量", "数据纯属虚构")
    bar_5.add("春季", attr, [randint(10, 100) for _ in range(6)])
    bar_5.add("夏季", attr, [randint(10, 100) for _ in range(6)])
    bar_5.add("秋季", attr, [randint(10, 100) for _ in range(6)])
    bar_5.add("冬季", attr, [randint(10, 100) for _ in range(6)], is_legend_show=True)

    timeline = Timeline(is_auto_play=True, timeline_bottom=0)
    timeline.add(bar_1, '2012 年')
    timeline.add(bar_2, '2013 年')
    timeline.add(bar_3, '2014 年')
    timeline.add(bar_4, '2015 年')
    timeline.add(bar_5, '2016 年')
    timeline.render()


def test_timeline_pie():

    attr = ["衬衫", "羊毛衫", "雪纺衫", "裤子", "高跟鞋", "袜子"]
    pie_1 = Pie("2012 年销量比例", "数据纯属虚构")
    pie_1.add("秋季", attr, [randint(10, 100) for _ in range(6)],
              is_label_show=True, radius=[30, 55], rosetype='radius')

    pie_2 = Pie("2013 年销量比例", "数据纯属虚构")
    pie_2.add("秋季", attr, [randint(10, 100) for _ in range(6)],
              is_label_show=True, radius=[30, 55], rosetype='radius')

    pie_3 = Pie("2014 年销量比例", "数据纯属虚构")
    pie_3.add("秋季", attr, [randint(10, 100) for _ in range(6)],
              is_label_show=True, radius=[30, 55], rosetype='radius')

    pie_4 = Pie("2015 年销量比例", "数据纯属虚构")
    pie_4.add("秋季", attr, [randint(10, 100) for _ in range(6)],
              is_label_show=True, radius=[30, 55], rosetype='radius')

    pie_5 = Pie("2016 年销量比例", "数据纯属虚构")
    pie_5.add("秋季", attr, [randint(10, 100) for _ in range(6)],
              is_label_show=True, radius=[30, 55], rosetype='radius')

    timeline = Timeline(is_auto_play=True, timeline_bottom=0)
    timeline.add(pie_1, '2012 年')
    timeline.add(pie_2, '2013 年')
    timeline.add(pie_3, '2014 年')
    timeline.add(pie_4, '2015 年')
    timeline.add(pie_5, '2016 年')
    timeline.show_config()
    timeline.render()


def test_timeline_bar_line():

    from pyecharts import Overlap
    attr = ["{}月".format(i) for i in range(1, 7)]
    bar = Bar("1 月份数据", "数据纯属虚构")
    bar.add("bar", attr, [randint(10, 50) for _ in range(6)])
    line = Line()
    line.add("line", attr, [randint(50, 80) for _ in range(6)])
    overlap = Overlap()
    overlap.add(bar)
    overlap.add(line)

    bar_1 = Bar("2 月份数据", "数据纯属虚构")
    bar_1.add("bar", attr, [randint(10, 50) for _ in range(6)])
    line_1 = Line()
    line_1.add("line", attr, [randint(50, 80) for _ in range(6)])
    overlap_1 = Overlap()
    overlap_1.add(bar_1)
    overlap_1.add(line_1)

    bar_2 = Bar("3 月份数据", "数据纯属虚构")
    bar_2.add("bar", attr, [randint(10, 50) for _ in range(6)])
    line_2 = Line()
    line_2.add("line", attr, [randint(50, 80) for _ in range(6)])
    overlap_2 = Overlap()
    overlap_2.add(bar_2)
    overlap_2.add(line_2)

    bar_3 = Bar("4 月份数据", "数据纯属虚构")
    bar_3.add("bar", attr, [randint(10, 50) for _ in range(6)])
    line_3 = Line()
    line_3.add("line", attr, [randint(50, 80) for _ in range(6)])
    overlap_3 = Overlap()
    overlap_3.add(bar_3)
    overlap_3.add(line_3)

    bar_4 = Bar("5 月份数据", "数据纯属虚构")
    bar_4.add("bar", attr, [randint(10, 50) for _ in range(6)])
    line_4 = Line()
    line_4.add("line", attr, [randint(50, 80) for _ in range(6)])
    overlap_4 = Overlap()
    overlap_4.add(bar_4)
    overlap_4.add(line_4)

    timeline = Timeline(timeline_bottom=0)
    timeline.add(overlap.chart, '1 月')
    timeline.add(overlap_1.chart, '2 月')
    timeline.add(overlap_2.chart, '3 月')
    timeline.add(overlap_3.chart, '4 月')
    timeline.add(overlap_4.chart, '5 月')
    timeline.render()



#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base
from pyecharts.option import get_all_options

class Radar(Base):
    """
    <<< Radar chart >>>
    Radar chart is mainly used to show multi-variable data,
    such as the analysis of a football player's varied attributes. It relies radar component.
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(Radar, self).__init__(title, subtitle, **kwargs)

    def config(self, schema=None,
               c_schema=None,
               shape="",
               rader_text_color="#000",
               **kwargs):
        """ config rader component options

        :param schema:
            The default radar map indicator, used to specify multiple dimensions in the radar map,
            will process the data into a dictionary of {name: xx, value: xx}
        :param c_schema:
            Indicator of radar chart, which is used to assign multiple variables(dimensions) in radar chart.
            name: Indicator's name.
            min: The maximum value of indicator. It is an optional configuration, but we recommend to set it manually.
            max: The maximum value of indicator. It is an optional configuration, but we recommend to set it manually.
        :param shape:
            Radar render type, in which 'polygon' and 'circle' are supported.
        :param rader_text_color:
            Radar chart data item font color
        :param kwargs:
        """
        chart = get_all_options(**kwargs)
        indicator = []
        if schema:
            for s in schema:
                _name, _max = s
                indicator.append({"name": _name, "max": _max})
        if c_schema:
            indicator = c_schema
        self._option.update(
            radar={
                "indicator": indicator,
                "shape": shape,
                "name": {"textStyle": {"color": rader_text_color}},
                "splitLine": chart['split_line'],
                "splitArea": chart['split_area'],
                "axisLine": chart['axis_line']}
        )

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, value, item_color=None, **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param value:
            data array of series, it is represented by a two-dimension array -> [[],[]]
        :param item_color:
            Specify a single legend color
        :param kwargs:
        """
        kwargs.update(flag=True, type='radar')
        chart = get_all_options(**kwargs)
        self._option.get('legend')[0].get('data').append(name)
        self._option.get('series').append({
            "type": "radar",
            "name": name,
            "data": value,
            "symbol": chart['symbol'],
            "itemStyle": {"normal": {"color": item_color}},
            "lineStyle": chart['line_style'],
            "areaStyle": {"normal": chart['area_style']},
        })
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
# coding=utf-8

from pyecharts import template


class Page(object):

    def __init__(self):
        self.__charts = []

    def add(self, achart_or_charts):
        """
        Append chart(s) to the rendering page

        :param achart_or_charts:
        :return:
        """
        if isinstance(achart_or_charts, list):
            self.__charts.extend(achart_or_charts)
        else:
            self.__charts.append(achart_or_charts)

    def render(self, path="render.html"):
        """
        Produce rendered charts in a html file

        :param path:
        :return:
        """
        template_name = "multicharts.html"
        chart_content = self.render_embed()
        tmp = template.JINJA2_ENV.get_template(template_name)
        html = tmp.render(multi_chart_content=chart_content)
        html = template.freeze_js(html)
        template.write_utf8_html_file(path, html)

    def render_embed(self):
        """
        Produce rendered charts in html for embedding purpose

        :return:
        """
        chart_content = ""
        for chart in self.__charts:
            chart_content += chart.render_embed()
        return chart_content



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from pyecharts import Liquid

def test_liquid():

    # liquid_0
    liquid = Liquid("水球图示例")
    liquid.add("Liquid", [0.6])
    liquid.show_config()
    liquid.render()

    # liquid_1
    liquid = Liquid("水球图示例")
    liquid.add("Liquid", [0.6, 0.5, 0.4, 0.3], is_liquid_outline_show=False)
    liquid.show_config()
    liquid.render()

    # liquid_2
    liquid = Liquid("水球图示例")
    liquid.add("Liquid", [0.6, 0.5, 0.4, 0.3], is_liquid_animation=False, shape='diamond')
    liquid.show_config()
    liquid.render()



#!/usr/bin/env python
# coding=utf-8


class Overlap(object):

    def __init__(self):
        self._chart = None

    def add(self, chart):
        """

        :param chart:
            chart instance
        :return:
        """
        if self._chart is None:
            self._chart = chart
        else:
            self.__custom(self.__get_series(chart))

    def __get_series(self, chart):
        """ Get chart series data

        :param chart:
            chart instance
        :return:
        """
        return (
            chart._option.get('legend')[0].get('data'),
            chart._option.get('series'),
        )

    def __custom(self, series):
        """ Appends the data for the series of the chart type

        :param series:
            series data
        """
        _name, _series = series
        for n in _name:
            self._chart._option.get('legend')[0].get('data').append(n)
        for s in _series:
            self._chart._option.get('series').append(s)

    def render(self, path="render.html"):
        """

        :param path:
        :return:
        """
        self._chart.render(path)

    def render_embed(self):
        """

        :return:
        """
        return self._chart.render_embed()

    def show_config(self):
        """

        :return:
        """
        import pprint
        return pprint.pprint(self._chart._option)

    @property
    def chart(self):
        """

        :return:
        """
        return self._chart

    def _repr_html_(self):
        """

        :return:
        """
        return self._chart._repr_html_()



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from pyecharts import EffectScatter

def test_effectscatter():

    # effectSactter_0
    v1 = [10, 20, 30, 40, 50, 60]
    v2 = [25, 20, 15, 10, 60, 33]
    es = EffectScatter("动态散点图示例")
    es.add("effectScatter", v1, v2)
    es.render()

    # effectScatter_1
    es = EffectScatter("动态散点图各种图形示例")
    es.add("", [10], [10], symbol_size=20, effect_scale=3.5, effect_period=3, symbol="pin")
    es.add("", [20], [20], symbol_size=12, effect_scale=4.5, effect_period=4, symbol="rect")
    es.add("", [30], [30], symbol_size=30, effect_scale=5.5, effect_period=5, symbol="roundRect")
    es.add("", [40], [40], symbol_size=10, effect_scale=6.5, effect_brushtype='fill', symbol="diamond")
    es.add("", [50], [50], symbol_size=16, effect_scale=5.5, effect_period=3, symbol="arrow")
    es.add("", [60], [60], symbol_size=6, effect_scale=2.5, effect_period=3, symbol="triangle")
    es.render()



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from pyecharts import Bar

def test_bar():

    # bar_0
    attr = ["衬衫", "羊毛衫", "雪纺衫", "裤子", "高跟鞋", "袜子"]
    v1 = [5, 20, 36, 10, 75, 90]
    v2 = [10, 25, 8, 60, 20, 80]
    bar = Bar("柱状图数据堆叠示例")
    bar.add("商家A", attr, v1, is_stack=True)
    bar.add("商家B", attr, v2, is_stack=True)
    bar.render()

    # bar_1
    bar = Bar("标记线和标记点示例")
    bar.add("商家A", attr, v1, mark_point=["average"])
    bar.add("商家B", attr, v2, mark_line=["min", "max"])
    bar.render()

    # bar_2
    bar = Bar("x 轴和 y 轴交换")
    bar.add("商家A", attr, v1)
    bar.add("商家B", attr, v2, is_convert=True)
    bar.render()

    # bar_3
    attr = ["{}月".format(i) for i in range(1, 13)]
    v1 = [2.0, 4.9, 7.0, 23.2, 25.6, 76.7, 135.6, 162.2, 32.6, 20.0, 6.4, 3.3]
    v2 = [2.6, 5.9, 9.0, 26.4, 28.7, 70.7, 175.6, 182.2, 48.7, 18.8, 6.0, 2.3]
    bar = Bar("柱状图示例")
    bar.add("蒸发量", attr, v1, mark_line=["average"], mark_point=["max", "min"])
    bar.add("降水量", attr, v2, mark_line=["average"], mark_point=["max", "min"],
            is_datazoom_show=True, datazoom_range=[20, 60], datazoom_type='inside')
    bar.show_config()
    bar.render()

    # bar_3_1
    bar = Bar("柱状图示例")
    bar.add("蒸发量", attr, v1, mark_line=["average"], mark_point=["max", "min"])
    bar.add("降水量", attr, v2, mark_line=["average"], mark_point=["max", "min"],
            is_datazoom_show=True, datazoom_range=[50, 80])
    bar.show_config()
    bar.render()

    # bar_4
    import random

    attr = ["{}天".format(i) for i in range(30)]
    v1 = [random.randint(1, 30) for _ in range(30)]
    bar = Bar("Bar - datazoom - slider 示例")
    bar.add("", attr, v1, is_label_show=True, is_datazoom_show=True)
    bar.show_config()
    bar.render()

    # bar_5
    attr = ["{}天".format(i) for i in range(30)]
    v1 = [random.randint(1, 30) for _ in range(30)]
    bar = Bar("Bar - datazoom - inside 示例")
    bar.add("", attr, v1, is_datazoom_show=True, datazoom_type='inside', datazoom_range=[10, 25])
    bar.show_config()
    bar.render()

    # bar_6
    attr = ["{}天".format(i) for i in range(20)]
    v1 = [random.randint(1, 20) for _ in range(20)]
    bar = Bar("坐标轴标签旋转示例")
    bar.add("", attr, v1, xaxis_interval=0, xaxis_rotate=30, yaxis_rotate=30)
    bar.show_config()
    bar.render()



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

import os
import re
import sys
import codecs
from jinja2 import Environment, FileSystemLoader

PY2 = sys.version_info[0] == 2

JS_PATTERN = re.compile(r'<!-- build -->(.*)<!-- endbuild -->',
                        re.IGNORECASE | re.MULTILINE | re.DOTALL)
JS_SRC_PATTERN = re.compile(r'src=\"(.*?)\"')


def freeze_js(html_content):
    """

    :param html_content:
    :return:
    """
    matches = JS_PATTERN.finditer(html_content)

    if not matches:
        return html_content

    for match in reversed(tuple(matches)):
        # JS file block
        src_matches = JS_SRC_PATTERN.findall(match.group(1))

        js_content = ""
        for src in src_matches:
            file_path = os.path.join(
                get_resource_dir('templates'), src.strip())

            with codecs.open(file_path, "r", "utf-8") as f:
                js_content += f.read() + '\n'
        # Replace matched string with inline JS
        fmt = '<script type="text/javascript">{}</script>'
        js_content = fmt.format(js_content)
        html_content = (html_content[:match.start()] + js_content +
                        html_content[match.end():])

    return html_content


CITY_NAME_PINYIN_MAP = {
    "广东": "guangdong",
    "安徽": "anhui",
    "澳门": "aomen",
    "北京": "beijing",
    "重庆": "chongqing",
    "福建": "fujian",
    "甘肃": "gansu",
    "广西": "guangxi",
    "贵州": "guizhou",
    "海南": "hainan",
    "河北": "hebei",
    "黑龙江": "heilongjiang",
    "河南": "henan",
    "湖北": "hubei",
    "湖南": "hunan",
    "江苏": "jiangsu",
    "江西": "jiangxi",
    "吉林": "jilin",
    "辽宁": "liaoning",
    "内蒙古": "neimenggu",
    "宁夏": "ningxia",
    "青海": "qinghai",
    "山东": "shandong",
    "上海": "shanghai",
    "山西": "shanxi",
    "四川": "sichuan",
    "台湾": "taiwan",
    "天津": "tianjin",
    "香港": "xianggang",
    "新疆": "xinjiang",
    "西藏": "xizang",
    "云南": "yunnan",
    "浙江": "zhejiang"
}


def get_resource_dir(folder):
    """

    :param folder:
    :return:
    """
    current_path = os.path.dirname(__file__)
    resource_path = os.path.join(current_path, folder)
    return resource_path


# Single Singleton Instance for jinja2
JINJA2_ENV = Environment(
    loader=FileSystemLoader(get_resource_dir('templates')),
    keep_trailing_newline=True,
    trim_blocks=True,
    lstrip_blocks=True)


def write_utf8_html_file(file_name, html_content):
    """

    :param file_name:
    :param html_content:
    :return:
    """
    if PY2:
        html = html_content.encode('utf-8')
        with open(file_name, "w+") as fout:
            fout.write(html)
    else:
        with open(file_name, "w+", encoding="utf-8") as fout:
            fout.write(html_content)



#!/usr/bin/env python
# coding=utf-8

from pyecharts.base import Base
from pyecharts.option import get_all_options


class Geo(Base):
    """
    Geographic coorinate system component.

    It is used to draw maps, which also supports scatter series, and line
    series.
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(Geo, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, attr, value,
              type="scatter",
              maptype='china',
              symbol_size=12,
              border_color="#111",
              geo_normal_color="#323c48",
              geo_emphasis_color="#2a333d",
              **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with
            legend, or updaing data and configuration with setOption.
        :param attr:
            name of attribute
        :param value:
            value of attribute
        :param type:
            chart type, it can be 'scatter', 'effectscatter', 'heatmap'
        :param maptype:
            type of map, it only supports 'china' temporarily.
        :param symbol_size:
            symbol size
        :param border_color:
            color of map border
        :param geo_normal_color:
            The color of the map area in normal state
        :param geo_emphasis_color:
            The color of the map area in emphasis state
        :param kwargs:
        """
        assert len(attr) == len(value)
        chart = get_all_options(**kwargs)
        _data = []
        for name, value in zip(attr, value):
            if name in self._geo_cities:
                city_coordinate = self._geo_cities.get(name)
                city_coordinate.append(value)
                _data.append({"name": name, "value": city_coordinate})
            else:
                print("%s coordinates is not found" % name)
        self._option.update(
            geo={
                "map": maptype,
                "label": {},
                "itemStyle": {"normal": {
                    "areaColor": geo_normal_color,
                    "borderColor": border_color},
                    "emphasis": {"areaColor": geo_emphasis_color}}
            })
        self._option.get('legend')[0].get('data').append(name)
        if type == "scatter":
            self._option.get('series').append({
                "type": type,
                "name": name,
                "coordinateSystem": 'geo',
                "symbol": chart['symbol'],
                "symbolSize": symbol_size,
                "data": _data,
                "label": chart['label'],
            })
        elif type == "effectScatter":
            self._option.get('series').append({
                "type": type,
                "name": name,
                "coordinateSystem": 'geo',
                "showEffectOn": "render",
                "rippleEffect": chart['effect'],
                "symbol": chart['symbol'],
                "symbolSize": symbol_size,
                "data": _data,
                "label": chart['label'],
            })
        elif type == "heatmap":
            self._option.get('series').append({
                "type": type,
                "name": name,
                "coordinateSystem": 'geo',
                "data": _data,
            })
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base
from pyecharts.option import get_all_options

class Graph(Base):
    """
    <<< Graph chart >>>
    Graph is a diagram to represent nodes and the links connecting nodes.
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(Graph, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, nodes, links, categories=None,
              is_focusnode=True,
              is_roam=True,
              is_rotatelabel=False,
              layout="force",
              edge_length=50,
              gravity=0.2,
              repulsion=50,
              **kwargs):
        """
        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param nodes:
            Relational nodes data
            name：Name of data item.     # required！！
            x：x value of node position.
            y：y value of node position.
            value：value of data item.
            category：Index of category which the data item belongs to.
            symbol：Symbol of node of this category.
                    Includes 'circle', 'rect', 'roundRect', 'triangle', 'diamond', 'pin', 'arrow'
            symbolSize：symbol size
        :param links:
            Relational data between nodes
            source：name of source node on edge      # required！！
            target：name of target node on edge      # required！！
            value：value of edge,It can be used in the force layout to map to the length of the edge
        :param categories:
            Name of category, which is used to correspond with legend and the content of tooltip.
        :param is_focusnode:
            Whether to focus/highlight the hover node and it's adjacencies.
        :param is_roam:
            Whether to enable mouse zooming and translating. false by default.
            If either zooming or translating is wanted,
            it can be set to 'scale' or 'move'. Otherwise, set it to be true to enable both.
        :param is_rotatelabel:
            Whether to rotate the label automatically.
        :param layout:
            Graph layout.
            'none': No any layout, use x, y provided in node as the position of node.
            'circular': Adopt circular layout, see the example Les Miserables.
            'force': Adopt force-directed layout, see the example Force,
                     the detail about configrations of layout are in graph.force
        :param edge_length:
            The distance between 2 nodes on edge. This distance is also affected by repulsion.
            It can be an array to represent the range of edge length.In this case edge with
            larger value will be shorter, which means two nodes are closer. And edge with smaller value will be longer.
        :param gravity:
            The gravity factor enforcing nodes approach to the center.
            The nodes will be closer to the center as the value becomes larger.
        :param repulsion:
            The repulsion factor between nodes. The repulsion will be stronger and the distance between 2 nodes
            becomes further as this value becomes larger.
            It can be an array to represent the range of repulsion.
            In this case larger value have larger repulsion and smaller value will have smaller repulsion.
        :param kwargs:
        """
        kwargs.update(type="graph")
        chart = get_all_options(**kwargs)
        if categories:
            for c in categories:
                self._option.get('legend')[0].get('data').append(c)
        self._option.get('series').append({
            "type": "graph",
            "name": name,
            "layout": layout,
            "symbol": chart['symbol'],
            "circular": {"rotateLabel": is_rotatelabel},
            "force": {"repulsion": repulsion,
                      "edgeLength": edge_length,
                      "gravity": gravity},
            "label": chart['label'],
            "lineStyle": chart['line_style'],
            "roam": is_roam,
            "focusNodeAdjacency": is_focusnode,
            "data": nodes,
            "categories": categories,
            "links":links,
        })
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from pyecharts import Scatter3D

def test_scatter3D():

    # scatter3D_0
    import random
    data = [[random.randint(0, 100), random.randint(0, 100), random.randint(0, 100)] for _ in range(80)]
    range_color = ['#313695', '#4575b4', '#74add1', '#abd9e9', '#e0f3f8', '#ffffbf',
                   '#fee090', '#fdae61', '#f46d43', '#d73027', '#a50026']
    scatter3D = Scatter3D("3D 散点图示例", width=1200, height=600)
    scatter3D.add("", data, is_visualmap=True, visual_range_color=range_color)
    scatter3D.show_config()
    scatter3D.render()



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

import random

fs = []

def collectfuncs(func):
    fs.append(func)
    return func


@collectfuncs
def label(type=None,
          is_emphasis=True,
          is_label_show=False,
          label_pos=None,
          label_text_color="#000",
          label_text_size=12,
          formatter=None,
          **kwargs):
    """ Text label of , to explain some data information about graphic item like value, name and so on.
        In ECharts 3, to make the configuration structure flatter,
        labelis taken to be at the same level with itemStyle, and has two status normal and emphasis as itemStyle does.

    :param type:
        Chart type
    :param is_emphasis:
        It specifies whether to show laebl in emphasis status.
    :param is_label_show:
        It specifies whether to show laebl in normal status.
    :param label_pos:
        Label position.It can be 'top', 'left', 'right', 'bottom', 'inside','outside'
    :param label_text_color:
        Label text color.
    :param label_text_size:
        Label font size.
    :param formatter:
        Data label formatter,it can be 'series', 'name', 'value', 'precent' 
    :param kwargs:
    :return:
    """
    if label_pos is None:
        label_pos = "outside" if type in ["pie", "graph"] else "top"
    _label = {
        "normal": {"show": is_label_show,
                   "position": label_pos,
                   "textStyle": {"color": label_text_color,
                                 "fontSize": label_text_size}},
        "emphasis": {"show": is_emphasis}
    }
    fmat = {"series": "{a} ", "name": "{b} ", "value": "{c} ", "percent": "{d}% "}
    if formatter is None:
        _formatter = "{b} {d}%" if type == "pie" else None
    else:
        _formatter = "".join([fmat.get(f) for f in formatter if fmat.get(f, None)])
    if type != "graph":
        _label.get("normal").update(formatter=_formatter)
    return _label


@collectfuncs
def color(colorlst=None,
          is_random=False,
          label_color=None,
          **kwargs):
    """

    :param colorlst:
        Global color list
    :param is_random:
        It specifies whether to random global color list.
    :param label_color:
        color append to global color list
    :param kwargs:
    :return:
    """
    if colorlst is None:
        colorlst = []
    if label_color:
        for color in reversed(list(label_color)):
            colorlst.insert(0, color)
    if is_random:
        random.shuffle(colorlst)
    return colorlst


@collectfuncs
def line_style(line_width=1,
               line_opacity=1,
               line_curve=0,
               line_type="solid",
               **kwargs):
    """

    :param line_width:
        Line width.
    :param line_opacity:
        Opacity of the component. Supports value from 0 to 1, and the component will not be drawn when set to 0.
    :param line_curve:
        Edge curvature, which supports value from 0 to 1. The larger the value, the greater the curvature. -> Graph
    :param line_type:
        Line type,it can be 'solid', 'dashed', 'dotted'
    :param kwargs:
    :return:
    """
    _line_style = {
        "normal": {"width": line_width,
                   "opacity": line_opacity,
                   "curveness": line_curve,
                   "type": line_type}
    }
    return _line_style


@collectfuncs
def split_line(is_splitline_show=True, **kwargs):
    """

    :param is_splitline_show:
        It specifies whether to show split line.
    :param kwargs:
    :return:
    """
    _split_line = {
        "show": is_splitline_show,
        "lineStyle": line_style(**kwargs)
    }
    return _split_line


@collectfuncs
def axis_line(is_axisline_show=True, **kwargs):
    """

    :param is_axisline_show:
        It specifies whether to show axis line.
    :param kwargs:
    :return:
    """
    _axis_line = {
        "show": is_axisline_show,
        "lineStyle": line_style(**kwargs)
    }
    return _axis_line


@collectfuncs
def split_area(is_area_show=True, **kwargs):
    """

    :param is_area_show:
        It specifies whether to show split area.
    :param kwargs:
    :return:
    """
    _split_area = {
        "show": is_area_show,
        "areaStyle": area_style(**kwargs)
    }
    return _split_area


@collectfuncs
def area_style(flag=False,
               area_opacity=None,
               area_color=None,
               **kwargs):
    """

    :param flag:
        chart type flag
    :param area_opacity:
        Opacity of the component. Supports value from 0 to 1, and the component will not be drawn when set to 0.
    :param area_color:
        Fill color.
    :param kwargs:
    :return:
    """
    if area_opacity is None:
        area_opacity = 0 if flag else 1
    _area_style = {
        "opacity": area_opacity,
        "color": area_color
    }
    return _area_style


@collectfuncs
def xy_axis(type=None,
            x_axis=None,
            xaxis_margin=8,
            xaxis_name_size=14,
            xaxis_name_gap=25,
            xaxis_name="",
            xaxis_name_pos="middle",
            xaxis_rotate=0,
            xaxis_min=None,
            xaxis_max=None,
            xaxis_type=None,
            xaxis_interval="auto",
            yaxis_margin=8,
            yaxis_name_size=14,
            yaxis_name_gap=25,
            yaxis_name="",
            yaxis_name_pos="middle",
            yaxis_rotate=0,
            yaxis_min=None,
            yaxis_max=None,
            yaxis_type=None,
            yaxis_interval="auto",
            yaxis_formatter="",
            is_convert=False,
            is_xaxislabel_align=False,
            is_yaxislabel_align=False,
            **kwargs):
    """

    :param type:
        Chart type
    :param x_axis:
        xAxis data
    :param xaxis_name_size:
        xaxis name font size
    :param xaxis_name_gap:
        Gap between axis name and xaxis line.
    :param xaxis_name:
        Name of xAxis
    :param xaxis_name_pos:
        Location of xAxis name.It can be 'start'，'middle'，'end'
    :param xaxis_rotate:
        Rotation degree of xaxis label, which is especially useful when there is no enough space for category axis.
        Rotation degree is from -90 to 90.
    :param xaxis_min:
        The minimun value of xaxis.
    :param xaxis_max:
        The maximun value of xaxis.
    :param xaxis_margin:
        The margin between the axis label and the xaxis line.
    :param xaxis_type:
        Type of xaxis
        'value' Numerical axis, suitable for continuous data.
        'category' Category axis, suitable for discrete category data. Data should only be set via data for this type.
        'time' Time axis, suitable for continuous time series data. As compared to value axis,
               it has a better formatting for time and a different tick calculation method. For example,
               it decides to use month, week, day or hour for tick based on the range of span.
        'log' Log axis, suitable for log data.
    :param xaxis_interval:
        The display interval of the axis scale label is valid in the category xaxis.
        By default, labels are displayed using labels that do not overlap the labels
        Set to 0 to force all labels to be displayed
        and label is one by one if setting as 1; If 2, it will be one label separates from each other, and so on.
    :param yaxis_interval:
        The display interval of the axis scale label is valid in the category yaxis.
        By default, labels are displayed using labels that do not overlap the labels
        Set to 0 to force all labels to be displayed
        and label is one by one if setting as 1; If 2, it will be one label separates from each other, and so on.
    :param yaxis_name:
        Name of yAxis
    :param yaxis_name_size:
        yaxis name font size
    :param yaxis_name_gap:
        Gap between axis name and yaxis line.
    :param yaxis_name_pos:
        Location of yAxis name.It can be 'start'，'middle'，'end'
    :param yaxis_rotate:
        Rotation degree of yaxis label, which is especially useful when there is no enough space for category axis.
        Rotation degree is from -90 to 90.
    :param yaxis_margin:
        The margin between the axis label and the yaxis line.
    :param yaxis_min:
        The minimun value of yaxis.
    :param yaxis_max:
        The maximun value of yaxis.
    :param yaxis_type:
        Type of yaxis
        'value' Numerical axis, suitable for continuous data.
        'category' Category axis, suitable for discrete category data. Data should only be set via data for this type.
        'time' Time axis, suitable for continuous time series data. As compared to value axis,
               it has a better formatting for time and a different tick calculation method. For example,
               it decides to use month, week, day or hour for tick based on the range of span.
        'log' Log axis, suitable for log data.
    :param is_convert:
        It specifies whether to convert xAxis and yAxis.
    :param yaxis_formatter:
        Formatter of axis label, which supports string template and callback function.
        example: '{value} kg'
    :param is_xaxislabel_align:
        whether align xaxis tick with label
    :param is_yaxislabel_align:
        whether align yaxis tick with label
    :param kwargs:
    :return:
    """
    _xAxis = {
        "name": xaxis_name,
        "nameLocation": xaxis_name_pos,
        "nameGap": xaxis_name_gap,
        "nameTextStyle": {"fontSize": xaxis_name_size},
        "axisLabel": {
            "interval": xaxis_interval,
            "rotate": xaxis_rotate,
            "margin": xaxis_margin
        },
        "axisTick": {
            "alignWithLabel": is_xaxislabel_align
        },
        "min": xaxis_min,
        "max": xaxis_max
    }
    _yAxis = {
        "name": yaxis_name,
        "nameLocation": yaxis_name_pos,
        "nameGap": yaxis_name_gap,
        "nameTextStyle": {"fontSize": yaxis_name_size},
        "axisLabel": {
            "formatter": "{value} " + yaxis_formatter,
            "rotate": yaxis_rotate,
            "interval": yaxis_interval,
            "margin": yaxis_margin
        },
        "axisTick": {
            "alignWithLabel": is_yaxislabel_align
        },
        "min": yaxis_min,
        "max": yaxis_max
    }

    if type == "scatter":
        if xaxis_type is None:
            xaxis_type = "value"
        if yaxis_type is None:
            yaxis_type = "value"
    else:       # line/bar
        if xaxis_type is None:
            xaxis_type = "category"
        if yaxis_type is None:
            yaxis_type = "value"

    if is_convert:
        xaxis_type, yaxis_type = yaxis_type, xaxis_type
        _xAxis.update(type=xaxis_type)
        _yAxis.update(data=x_axis, type=yaxis_type)
    else:
        _xAxis.update(data=x_axis, type=xaxis_type)
        _yAxis.update(type=yaxis_type)

    if type == "candlestick":
        _xAxis.update(scale=True, boundaryGap=False)
        _yAxis.update(scale=True, splitArea={"show": True})

    return [_xAxis], [_yAxis]


def _mark(data,
          mark_point_symbol='pin',
          mark_point_symbolsize=50,
          mark_point_textcolor='#fff',
          _is_markline=False,
          **kwargs):
    """

    :param data:
        mark data, it can be 'min', 'max', 'average'
    :param mark_point_symbol:
        mark symbol, it cna be 'circle', 'rect', 'roundRect', 'triangle', 'diamond', 'pin', 'arrow'
    :param mark_point_symbolsize:
        mark symbol size
    :param mark_point_textcolor:
        mark point text color
    :param _is_markline:
        It specifies whether is markline or not.
    :return:
    """
    mark = {"data": []}
    if data:
        for d in list(data):
            _type, _name = "", ""
            if "max" in d:
                _type, _name = "max", "最大值"
            elif "min" in d:
                _type, _name = "min", "最小值"
            elif "average" in d:
                _type, _name = "average", "平均値"
            if _is_markline:
                _m = {
                    "type": _type,
                    "name": _name,
                }
            else:
                _m = {
                    "type": _type,
                    "name": _name,
                    "symbol": mark_point_symbol,
                    "symbolSize": mark_point_symbolsize,
                    "label": {"normal": {"textStyle": {"color": mark_point_textcolor}}}
                }
            if type:
                mark.get("data").append(_m)
    return mark


@collectfuncs
def mark_point(mark_point=None, **kwargs):
    """

    :param mark_point:
        mark point data, it can be 'min', 'max', 'average'
    :param kwargs:
    :return:
    """
    return _mark(mark_point, **kwargs)


@collectfuncs
def mark_line(mark_line=None, **kwargs):
    """

    :param mark_line:
        mark line data,it can be 'min', 'max', 'average'
    :param kwargs:
    :return:
    """
    return _mark(mark_line, _is_markline=True)


@collectfuncs
def legend(is_legend_show=True,
           legend_orient="horizontal",
           legend_pos="center",
           legend_top='top',
           legend_selectedmode='multiple',
           **kwargs):
    """ Legend component.
        Legend component shows symbol, color and name of different series.
        You can click legends to toggle displaying series in the chart.
        In ECharts 3, a single echarts instance may contain multiple legend components,
        which makes it easier for the layout of multiple legend components.
    :param is_legend_show:
        It specifies whether to show the legend component.
    :param legend_orient:
        The layout orientation of legend.It can be 'horizontal', 'vertical'
    :param legend_pos:
        Distance between legend component and the left side of the container.
        legend_pos value can be instant pixel value like 20;
        it can also be percentage value relative to container width like '20%';
        and it can also be 'left', 'center', or 'right'.
    :param legend_top:
        Distance between legend component and the top side of the container.
        legend_top value can be instant pixel value like 20;
        it can also be percentage value relative to container width like '20%';
        and it can also be 'top', 'middle', or 'bottom'.
    :param legend_selectedmode:
        State table of selected legend. 'single' or 'multiple'
    :param kwargs:
    :return:
    """
    _legend = {
        "selectedMode": legend_selectedmode,
        "show": is_legend_show,
        "left": legend_pos,
        "top": legend_top,
        "orient": legend_orient
    }
    return _legend


@collectfuncs
def visual_map(visual_type='color',
               visual_range=None,
               visual_text_color=None,
               visual_range_text=None,
               visual_range_color=None,
               visual_range_size=None,
               visual_orient='vertical',
               visual_pos="left",
               visual_top="bottom",
               is_calculable=True,
               **kwargs):
    """ visualMap is a type of component for visual encoding, which maps the data to visual channels

    :param visual_type:
        visual map type, 'color' or 'size'
        color: For visual channel color, array is used, like: ['#333', '#78ab23', 'blue'],
        which means a color ribbon is formed based on the three color stops,and dataValues will be mapped to the ribbon.
        size: For visual channel size, array is used, like: [20, 50],
        which means a size ribbon is formed based on the two value stops, and dataValues will be mapped to the ribbon.
    :param visual_range:
        pecify the min and max dataValue for the visualMap component.
    :param visual_text_color:
        visualMap text color.
    :param visual_range_text:
        The label text on both ends, such as ['High', 'Low']
    :param visual_range_size:
        For visual channel size, array is used, like: [20, 50].
    :param visual_range_color:
        For visual channel color, array is used, like: ['#333', '#78ab23', 'blue'].
    :param visual_orient:
        How to layout the visualMap component, 'horizontal' or 'vertical'.
    :param visual_pos:
        Distance between visualMap component and the left side of the container.
        visual_pos value can be instant pixel value like 20;
        it can also be percentage value relative to container width like '20%';
        and it can also be 'left', 'center', or 'right'.
    :param visual_top:
        Distance between visualMap component and the top side of the container.
        visual_top value can be instant pixel value like 20;
        it can also be percentage value relative to container width like '20%';
        and it can also be 'top', 'middle', or 'bottom'.
    :param is_calculable:
        Whether show handles, which can be dragged to adjust "selected range".
    :param kwargs:
    :return:
    """
    # defalut min and max value of visual_range is [0, 100]
    _min, _max = 0, 100
    if visual_range:
        if len(visual_range) == 2:
            _min, _max = visual_range

    # defalut label text on both ends is ['low', 'high']
    _tlow, _thigh = "low", "high"
    if visual_range_text:
        if len(visual_range_text) == 2:
            _tlow, _thigh = visual_range_text

    _inrange_op = {}
    if visual_type == 'color':
        range_color = ['#50a3ba', '#eac763', '#d94e5d']
        if visual_range_color:
            if len(visual_range_color) >= 2:
                range_color = visual_range_color
        _inrange_op.update(color=range_color)

    if visual_type == 'size':
        range_size = [20, 50]
        if visual_range_size:
            if len(visual_range_size) >= 2:
                range_size = visual_range_size
        _inrange_op.update(symbolSize=range_size)

    _visual_map = {
        "type": "continuous",
        "min": _min,
        "max": _max,
        "text": [_thigh, _tlow],
        "textStyle": {"color": visual_text_color},
        "inRange": _inrange_op,
        "calculable": is_calculable,
        "orient": visual_orient,
        "left": visual_pos,
        "top": visual_top
    }
    return _visual_map


def gen_color():
    """ random generation color -> WordCloud

    :return:
    """
    return "rgb(%s,%s,%s)" % (random.randint(0, 160),
                              random.randint(0, 160),
                              random.randint(0, 160))


@collectfuncs
def symbol(type=None, symbol="", **kwargs):
    """

    :param symbol:
        symbol type, it can be 'rect', 'roundRect', 'triangle', 'diamond', 'pin', 'arrow'.
    :param kwargs:
    :return:
    """
    if symbol is None:  # Radar
        symbol = 'none'
    elif type == "line" and symbol == "":  # Line
        symbol = "emptyCircle"
    elif symbol not in ('rect', 'roundRect', 'triangle', 'diamond', 'pin', 'arrow'):
        symbol = 'circle'
    return symbol


@collectfuncs
def effect(effect_brushtype="stroke",
           effect_scale=2.5,
           effect_period=4,
           **kwargs):
    """

    :param effect_brushtype:
        The brush type for ripples. options: 'stroke' and 'fill'.
    :param effect_scale:
        The maximum zooming scale of ripples in animation.
    :param effect_period:
        The duration of animation.
    :param kwargs:
    :return:
    """
    _effect = {
        "brushType": effect_brushtype,
        "scale": effect_scale,
        "period":effect_period
    }
    return _effect


@collectfuncs
def datazoom(is_datazoom_show=False,
             datazoom_type='slider',
             datazoom_range=None,
             datazoom_orient='horizontal',
             **kwargs):
    """

    :param is_datazoom_show:
        It specifies whether to use the datazoom component.
    :param datazoom_type:
        datazoom type, 'slider' or 'inside'
    :param datazoom_range:
        The range percentage of the window out of the data extent, in the range of 0 ~ 100.
    :param datazoom_orient:
        Specify whether the layout of dataZoom component is horizontal or vertical.'horizontal' or 'vertical'
        What's more,it indicates whether the horizontal axis or vertical axis is controlled
        by default in catesian coordinate system.
    :param kwargs:
    :return:
    """
    _min, _max = 50, 100
    if datazoom_range:
        if len(datazoom_range) == 2:
            _min, _max = datazoom_range
    if datazoom_type not in ("slider", "inside"):
        datazoom_type = "slider"
    _datazoom = {
        "show": is_datazoom_show,
        "type": datazoom_type,
        "start": _min,
        "end": _max,
        "orient": datazoom_orient
    }
    return [_datazoom]


@collectfuncs
def grid(grid_width=None,
         grid_height=None,
         grid_top=None,
         grid_bottom=None,
         grid_left=None,
         grid_right=None,
         **kwargs):
    """
    :param series:
        other chart series data
    :param grid_width:
        Width of grid component. Adaptive by default.
    :param grid_height:
        Height of grid component. Adaptive by default.
    :param grid_top:
        Distance between grid component and the top side of the container.
        grid_top value can be instant pixel value like 20;
        it can also be percentage value relative to container width like '20%';
        and it can also be 'top', 'middle', or 'bottom'.
        If the grid_top value is set to be 'top', 'middle', or 'bottom',
        then the component will be aligned automatically based on position.
    :param grid_bottom:
        Distance between grid component and the bottom side of the container.
        grid_bottom value can be instant pixel value like 20;
        it can also be percentage value relative to container width like '20%'.
    :param grid_left:
        Distance between grid component and the left side of the container.
        grid_left value can be instant pixel value like 20;
        it can also be percentage value relative to container width like '20%';
        and it can also be 'left', 'center', or 'right'.
        If the grid_left value is set to be 'left', 'center', or 'right',
        then the component will be aligned automatically based on position.
    :param grid_right:
        Distance between grid component and the right side of the container.
        grid_right value can be instant pixel value like 20;
        it can also be percentage value relative to container width like '20%'.
    :return:
    """
    _grid = {}
    if grid_width is not None:
        _grid.update(width=grid_width)
    if grid_height is not None:
        _grid.update(height=grid_height)
    if grid_top is not None:
        _grid.update(top=grid_top)
    if grid_bottom is not None:
        _grid.update(bottom=grid_bottom)
    if grid_left is not None:
        _grid.update(left=grid_left)
    if grid_right is not None:
        _grid.update(right=grid_right)
    return _grid


@collectfuncs
def grid3D(grid3D_width=100,
           grid3D_height=100,
           grid3D_depth=100,
           grid3D_rotate_speed=10,
           grid3D_rotate_sensitivity=1,
           is_grid3D_rotate=False,
           **kwargs):
    """

    :param grid3D_width:
        3D axis width
    :param grid3D_height:
        3D axis height
    :param grid3D_depth:
        3D axis depth
    :param grid3D_rotate_speed:
        3D charts rotate speed
    :param is_grid3D_rotate:
        whether rotate 3D charts
    :param grid3D_rotate_sensitivity:
        3D charts rotete sensitivity, The greater the value, the more sensitive.
    :param kwargs:
    :return:
    """
    _grid3D = {
        "boxWidth": grid3D_width,
        "boxHeight": grid3D_height,
        "boxDepth": grid3D_depth,
        "viewControl": {
            "autoRotate": is_grid3D_rotate,
            "autoRotateSpeed": grid3D_rotate_speed,
            "rotateSensitivity": grid3D_rotate_sensitivity
        }
    }
    return _grid3D


@collectfuncs
def xaxis3D(xaxis3d_type=None,
            xaxis3d_name="",
            xaxis3d_name_size=16,
            xaxis3d_name_gap=20,
            xaxis3d_min=None,
            xaxis3d_max=None,
            xaxis3d_interval="auto",
            xaxis3d_margin=8,
            **kwargs):
    """

    :param xaxis3d_type:
        Type of 3d xaxis
    :param xaxis3d_name:
        Name of 3d xaxis
    :param xaxis3d_name_size:
        3d xaxis name font size
    :param xaxis3d_name_gap:
        Gap between axis name and 3d xaxis line.
    :param xaxis3d_min:
        The minimun value of 3d xaxis.
    :param xaxis3d_max:
        The maximun value of 3d xaxis.
    :param xaxis3d_interval:
        The display interval of the axis scale label is valid in the category 3d xaxis.
        By default, labels are displayed using labels that do not overlap the labels
        Set to 0 to force all labels to be displayed
        and label is one by one if setting as 1; If 2, it will be one label separates from each other, and so on.
    :param xaxis3d_margin:
        The margin between the axis label and the 3d xaxis line.
    :return:
    """
    _xaxis3D = {
        "name": xaxis3d_name,
        "nameGap": xaxis3d_name_gap,
        "nameTextStyle": {"fontSize": xaxis3d_name_size},
        "type": xaxis3d_type,
        "min": xaxis3d_min,
        "max": xaxis3d_max,
        "axisLabel": {
            "margin": xaxis3d_margin,
            "interval": xaxis3d_interval
        }
    }
    return _xaxis3D


@collectfuncs
def yaxis3D(yaxis3d_type=None,
            yaxis3d_name="",
            yaxis3d_name_size=16,
            yaxis3d_name_gap=20,
            yaxis3d_min=None,
            yaxis3d_max=None,
            yaxis3d_interval="auto",
            yaxis3d_margin=8,
            **kwargs):
    """

    :param yaxis3d_type:
        Type of 3d yaxis
    :param yaxis3d_name:
        Name of 3d yaxis
    :param yaxis3d_name_size:
        3d yaxis name font size
    :param yaxis3d_name_gap:
        Gap between axis name and 3d yaxis line.
    :param yaxis3d_min:
        The minimun value of 3d yaxis.
    :param yaxis3d_max:
        The maximun value of 3d yaxis.
    :param yaxis3d_interval:
        The display interval of the axis scale label is valid in the category 3d yaxis.
        By default, labels are displayed using labels that do not overlap the labels
        Set to 0 to force all labels to be displayed
        and label is one by one if setting as 1; If 2, it will be one label separates from each other, and so on.
    :param yaxis3d_margin:
        The margin between the axis label and the 3d yaxis line.
    :return:
    """
    _yaxis3D = {
        "name": yaxis3d_name,
        "nameGap": yaxis3d_name_gap,
        "nameTextStyle": {"fontSize": yaxis3d_name_size},
        "type": yaxis3d_type,
        "min": yaxis3d_min,
        "max": yaxis3d_max,
        "axisLabel": {
            "margin": yaxis3d_margin,
            "interval": yaxis3d_interval
        }
    }
    return _yaxis3D


@collectfuncs
def zaxis3D(zaxis3d_type=None,
            zaxis3d_name="",
            zaxis3d_name_size=16,
            zaxis3d_name_gap=20,
            zaxis3d_min=None,
            zaxis3d_max=None,
            zaxis3d_margin=8,
            **kwargs):
    """

    :param zaxis3d_type:
        Type of 3d zaxis
    :param zaxis3d_name:
        Name of 3d zaxis
    :param zaxis3d_name_size:
        3d zaxis name font size
    :param zaxis3d_name_gap:
        Gap between axis name and 3d zaxis line.
    :param zaxis3d_min:
        The minimun value of 3d zaxis.
    :param zaxis3d_max:
        The maximun value of 3d zaxis.
    :param zaxis3d_margin:
        The margin between the axis label and the 3d zaxis line.
    :return:
    """
    _zaxis3D = {
        "name": zaxis3d_name,
        "nameGap": zaxis3d_name_gap,
        "nameTextStyle": {"fontSize": zaxis3d_name_size},
        "type": zaxis3d_type,
        "min": zaxis3d_min,
        "max": zaxis3d_max,
        "axisLabel": {
            "margin": zaxis3d_margin,
        }
    }
    return _zaxis3D


def get_all_options(**kwargs):
    """ Return all options of charts

    :param kwargs:
    :return:
    """
    _funcs = {}
    for f in fs:
        _funcs[f.__name__] = f(**kwargs)
    return _funcs



#!/usr/bin/env python
#coding=utf-8

from pyecharts.charts.scatter import Scatter
from pyecharts.option import get_all_options

class EffectScatter(Scatter):
    """
    <<< EffectScatter chart >>>
    The scatter graph with ripple animation. The special animation effect can visually highlights some data.
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(EffectScatter, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, x_value, y_value,
              symbol_size=10,
              **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param x_axis:
            data of xAxis
        :param y_axis:
            data of yAxis
        :param symbol_size:
            symbol size
        :param kwargs:
        """
        assert len(x_value) == len(y_value)
        kwargs.update(type="scatter")
        chart = get_all_options(**kwargs)
        xaxis, yaxis = chart['xy_axis']
        self._option.update(xAxis=xaxis, yAxis=yaxis)
        self._option.get('legend')[0].get('data').append(name)
        self._option.get('series').append({
            "type": "effectScatter",
            "name": name,
            "showEffectOn":"render",
            "rippleEffect": chart['effect'],
            "symbol": chart['symbol'],
            "symbolSize": symbol_size,
            "data": [list(z) for z in zip(x_value, y_value)],
            "label": chart['label'],
            "indexflag": self._option.get('_index_flag')
        })
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base

class Gauge(Base):
    """
    <<< Gauge chart >>>
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(Gauge, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, attr, value,
              scale_range=None,
              angle_range=None,
              **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param attr:
            name of attribute
        :param value:
            value of attribute
        :param scale_range:
            data range of guage
        :param angle_range:
            angle range of guage
            The direct right side of circle center is 0 degree,
            the right above it is 90 degree, the direct left side of it is 180 degree.
        :param kwargs:
        """
        # default data range is [0, 100]
        _min, _max = 0, 100
        if scale_range:
            if len(scale_range) == 2:
                _min, _max = scale_range

        # defalut angle range is [225, -45]
        _start, _end = 225, -45
        if angle_range:
            if len(angle_range) == 2:
                _start, _end = angle_range

        self._option.get('legend')[0].get('data').append(name)
        self._option.get('series').append({
            "type": "gauge",
            "detail": {"formatter": '{value}%'},
            "name": name,
            "min": _min,
            "max": _max,
            "startAngle": _start,
            "endAngle": _end,
            "data": [{"value": value, "name": attr}]
        })
        self._option.update(tooltip={"formatter": "{a} <br/>{b} : {c}%"})
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from pyecharts import Line

def test_line():

    # line_0
    attr = ["衬衫", "羊毛衫", "雪纺衫", "裤子", "高跟鞋", "袜子"]
    v1 = [5, 20, 36, 10, 10, 100]
    v2 = [55, 60, 16, 20, 15, 80]
    line = Line("折线图示例")
    line.add("商家A", attr, v1, mark_point=["average"])
    line.add("商家B", attr, v2, is_smooth=True, mark_line=["max", "average"])
    line.show_config()
    line.render()

    # line_0_1
    line = Line("折线图示例")
    line.add("商家A", attr, v1, mark_point=["average", "max", "min"],
             mark_point_symbol='diamond', mark_point_textcolor='#40ff27')
    line.add("商家B", attr, v2, mark_point=["average", "max", "min"],
             mark_point_symbol='arrow', mark_point_symbolsize=40)
    line.show_config()
    line.render()

    # line_1
    line = Line("折线图-数据堆叠示例")
    line.add("商家A", attr, v1, is_stack=True, is_label_show=True)
    line.add("商家B", attr, v2, is_stack=True, is_label_show=True)
    line.show_config()
    line.render()

    # line_2
    line = Line("折线图-阶梯图示例")
    line.add("商家A", attr, v1, is_step=True, is_label_show=True)
    line.show_config()
    line.render()

    # line_3
    line = Line("折线图-面积图示例")
    line.add("商家A", attr, v1, is_fill=True, line_opacity=0.2, area_opacity=0.4, symbol=None)
    line.add("商家B", attr, v2, is_fill=True, area_color='#000', area_opacity=0.3, is_smooth=True)
    line.show_config()
    line.render()

    # line_3_1
    import math, random
    line = Line("折线图示例")
    line.add("商家A", attr, [math.log10(random.randint(1, 99999)) for _ in range(6)])
    line.add("商家B", attr, [math.log10(random.randint(1, 99999999)) for _ in range(6)], yaxis_type="log")
    line.show_config()
    line.render()

    # line_4
    attr = ['周一', '周二', '周三', '周四', '周五', '周六', '周日']
    line = Line("折线图示例")
    line.add("最高气温", attr, [11, 11, 15, 13, 12, 13, 10], mark_point=["max", "min"], mark_line=["average"])
    line.add("最低气温", attr, [1, -2, 2, 5, 3, 2, 0], mark_point=["max", "min"], mark_line=["average"])
    line.show_config()
    line.render()



#!/usr/bin/env python
# coding=utf-8
from __future__ import unicode_literals

import json

import pandas as pd
import numpy as np

from pyecharts import Bar


def test_embed_option():

    title = "柱状图数据堆叠示例"
    attr = ["衬衫", "羊毛衫", "雪纺衫", "裤子", "高跟鞋", "袜子"]
    v1 = [5, 20, 36, 10, 75, 90]
    v2 = [10, 25, 8, 60, 20, 80]
    bar = Bar(title)
    bar.add("商家A", attr, v1, is_stack=True)
    bar.add("商家B", attr, v2, is_stack=True)
    html = bar.render_embed()
    json_encoded_title = json.dumps(title)
    assert json_encoded_title in html
    assert "<html>" not in html
    assert "<body>" not in html


def test_numpy_array():

    title = "柱状图数据堆叠示例"
    attr = ["衬衫", "羊毛衫", "雪纺衫", "裤子", "高跟鞋", "袜子"]
    v1 = np.array([5, 20, 36, 10, 75, 90])
    bar = Bar(title)
    bar.add("商家A", attr, v1, is_stack=True)
    html = bar.render_embed()
    json_encoded_title = json.dumps(title)
    assert json_encoded_title in html


def test_pandas_dataframe():

    title = 'Bar chart'
    index = pd.date_range('3/8/2017', periods=6, freq='M')
    df1 = pd.DataFrame(np.random.randn(6), index=index)
    df2 = pd.DataFrame(np.random.randn(6), index=index)

    dtvalue1 = [i[0] for i in df1.values]
    dtvalue2 = [i[0] for i in df2.values]

    bar = Bar(title, 'Profit and loss situation')
    bar.add('profit', df1.index, dtvalue1)
    bar.add('loss', df2.index, dtvalue2)
    html = bar.render_embed()
    assert title in html



#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base
from pyecharts.option import get_all_options

class Map(Base):
    """
    <<< Map chart >>>
    Map is maily used in the visulization of geographic area data,
    which can be used with visualMap component to visualize the datas such as population distribution
    density in diffrent areas.
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(Map, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, attr, value,
              is_roam=True,
              maptype='china',
              **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param attr:
            name of attribute
        :param value:
            value of attribute
        :param is_roam:
            Whether to enable mouse zooming and translating. false by default.
            If either zooming or translating is wanted,
            it can be set to 'scale' or 'move'. Otherwise, set it to be true to enable both.
        :param maptype:
            type of map, it supports
            china、world、...
        :param kwargs:
        """
        chart = get_all_options(**kwargs)
        assert len(attr) == len(value)
        _data = []
        for data in zip(attr, value):
            _name, _value = data
            _data.append({"name": _name, "value": _value})
        self._option.get('legend')[0].get('data').append(name)
        self._option.get('series').append({
            "type": "map",
            "name": name,
            "symbol": chart['symbol'],
            "mapType": maptype,
            "data": _data,
            "roam": is_roam
        })
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base
from pyecharts.option import get_all_options

class Polar(Base):
    """
    <<< Polar component >>>
    Polar coordinate can be used in scatter and line chart. Every polar coordinate has an angleAxis and a radiusAxis.
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(Polar, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, data,
              angle_data=None,
              radius_data=None,
              type='line',
              symbol_size=4,
              start_angle=90,
              rotate_step=0,
              boundary_gap=True,
              clockwise=True,
              is_stack=False,
              axis_range=None,
              is_angleaxis_show=True,
              is_radiusaxis_show=True,
              **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param data:
            data of polar, [Polar radius, Polar angle, [value]]
            it is represented by a two-dimension array -> [[],[]]
        :param angle_data:
            Category data for angle, available in type: 'category' axis.
        :param radius_data:
            Category data for radius, available in type: 'category' axis.
        :param type:
            chart type，it can be 'scatter', 'effectScatter', 'barAngle', 'barRadius'
        :param symbol_size:
            symbol size
        :param start_angle:
            Starting angle of axis. 90 degrees by default, standing for top position of center.
            0 degree stands for right position of center.
        :param rotate_step:
            Rotation degree of axis label, which is especially useful when there is no enough space for category axis.
            Rotation degree is from -90 to 90.
        :param boundary_gap:
            The boundary gap on both sides of a coordinate axis.
            The setting and behavior of category axes and non-category axes are different.
            The boundaryGap of category axis can be set to either true or false.
            Default value is set to be true, in which case axisTick is served only as a separation line,
            and labels and data appear only in the center part of two axis ticks, which is called band.
        :param clockwise:
            Whether the positive position of axis is in clockwise. True for clockwise by default.
        :param is_stack:
            It specifies whether to stack category axis.
        :param axis_range:
            axis scale range
        :param is_angleaxis_show:
            whether show angle axis.
        :param is_radiusaxis_show:
            whether show radius axis.
        :param kwargs:
        """
        chart = get_all_options(**kwargs)
        polar_type = 'value' if type == "line" else "category"
        is_stack = "stack" if is_stack else ""
        self._option.get('legend')[0].get('data').append(name)

        # By defalut, axis scale range is [None, None]
        _amin, _amax = None, None
        if axis_range:
            if len(axis_range) == 2:
                _amin, _amax = axis_range
        _area_style = {"normal": chart['area_style']}
        if kwargs.get('area_color', None) is None:
            _area_style = None

        if type in ("scatter", "line"):
            self._option.get('series').append({
                "type": type,
                "name": name,
                "coordinateSystem": 'polar',
                "symbol": chart['symbol'],
                "symbolSize": symbol_size,
                "data": data,
                "label": chart['label'],
                "areaStyle": _area_style
            })
        elif type == "effectScatter":
            self._option.get('series').append({
                "type": type,
                "name": name,
                "coordinateSystem": 'polar',
                "showEffectOn": "render",
                "rippleEffect": chart['effect'],
                "symbol": chart['symbol'],
                "symbolSize": symbol_size,
                "data": data,
                "label": chart['label'],
            })
        elif type == "barRadius":
            self._option.get('series').append({
                "type": "bar",
                "stack": is_stack,
                "name": name,
                "coordinateSystem": 'polar',
                "data": data,
            })
            self._option.update(angleAxis={})
            self._option.update(
                radiusAxis={
                    "type": polar_type,
                    "data": radius_data,
                    "z": 50,
                })
        elif type == "barAngle":
            self._option.get('series').append({
                "type": "bar",
                "stack": is_stack,
                "name": name,
                "coordinateSystem": 'polar',
                "data": data,
            })
            self._option.update(radiusAxis={})
            self._option.update(
                angleAxis={
                    "type": polar_type,
                    "data": radius_data,
                    "z": 50
                })

        if type not in ("barAngle", "barRadius"):
            self._option.update(
                angleAxis={
                    "show":is_angleaxis_show,
                    "type": polar_type,
                    "data": angle_data,
                    "clockwise": clockwise,
                    "startAngle": start_angle,
                    "boundaryGap": boundary_gap,
                    "splitLine": chart['split_line'],
                    "axisLine": chart['axis_line']
                }
            )
            self._option.update(
                radiusAxis={
                    "show": is_radiusaxis_show,
                    "type": polar_type,
                    "data": radius_data,
                    "min": _amin,
                    "max": _amax,
                    "axisLine": chart['axis_line'],
                    "axisLabel": {"rotate": rotate_step}
                }
            )
        self._option.update(polar={})
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
#coding=utf-8

from setuptools import setup
from setuptools.command.install import install

try:
    from jupyterpip import cmdclass
except:
    import pip, importlib
    pip.main(['install', 'jupyter-pip']); cmdclass = importlib.import_module('jupyterpip').cmdclass


__title__ = 'pyecharts'
__description__ = 'Python echarts, make charting easier'
__url__ = 'https://github.com/chenjiandongx/pyecharts'
__version__ = '0.1.9.5'
__author__ = 'chenjiandongx'
__author_email__ = 'chenjiandongx@qq.com'
__license__ = 'MIT'
__requires__ = ['pillow', 'jinja2', 'future', 'jupyter-pip']
__packages__ = ['pyecharts', 'pyecharts/charts', 'pyecharts/custom']
__keywords__ = ['echarts', 'charts']
__jupyter_echarts__ = 'pyecharts/templates/js/echarts'


setup(
    name=__title__,
    version=__version__,
    description=__description__,
    url=__url__,
    author=__author__,
    author_email=__author_email__,
    license=__license__,
    packages=__packages__,
    keywords=__keywords__,
    install_requires=__requires__,
    zip_safe=False,
    include_package_data=True,
    cmdclass=cmdclass(__jupyter_echarts__, enable="echarts/main"),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries'
    ]
)



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

from pyecharts import Map

def test_map():

    # map_0
    value = [155, 10, 66, 78]
    attr = ["福建", "山东", "北京", "上海"]
    map = Map("全国地图示例", width=1200, height=600)
    map.add("", attr, value, maptype='china')
    map.show_config()
    map.render()

    # map_1
    value = [155, 10, 66, 78, 33, 80, 190, 53, 49.6]
    attr = ["福建", "山东", "北京", "上海", "甘肃", "新疆", "河南", "广西", "西藏"]
    map = Map("Map 结合 VisualMap 示例", width=1200, height=600)
    map.add("", attr, value, maptype='china', is_visualmap=True, visual_text_color='#000')
    map.show_config()
    map.render()

    # map_2
    value = [20, 190, 253, 77, 65]
    attr = ['汕头市', '汕尾市', '揭阳市', '阳江市', '肇庆市']
    map = Map("广东地图示例", width=1200, height=600)
    map.add("", attr, value, maptype='广东', is_visualmap=True, visual_text_color='#000')
    map.show_config()
    map.render()

    # map_3
    value = [95.1, 23.2, 43.3, 66.4, 88.5]
    attr= ["China", "Canada", "Brazil", "Russia", "United States"]
    map = Map("世界地图示例", width=1200, height=600)
    map.add("", attr, value, maptype="world", is_visualmap=True, visual_text_color='#000')
    map.render()



#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base
from pyecharts.option import get_all_options

class HeatMap(Base):
    """
    <<< HeatMap chart >>>
    Heat map mainly use colors to represent values, which must be used along with visualMap component.
    It can be used in either rectangular coordinate or geographic coordinate.
    But the behaviour on them are quite different. Rectangular coordinate must have two catagories to use it.
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(HeatMap, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, x_axis, y_axis, data, **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param x_axis:
            data of xAxis, it must be catagory axis.
        :param y_axis:
            data of yAxis, it must be catagory axis.
        :param data:
            data array of series, it is represented by a two-dimension array -> [[],[]]
        :param kwargs:
        :return:
        """
        chart = get_all_options(**kwargs)
        self._option.get('legend')[0].get('data').append(name)
        self._option.update(
            xAxis=[{
                "type": "category",
                "data": x_axis,
                "splitArea": {"show": True},
            }],
            yAxis=[{
                "type": "category",
                "data": y_axis,
                "splitArea": {"show": True}
            }])
        self._option.get('series').append({
            "type": "heatmap",
            "name": name,
            "data": data,
            "label": chart['label'],
            "indexflag": self._option.get('_index_flag')
        })
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
#coding=utf-8
from __future__ import unicode_literals

import os
import sys
from pyecharts import Graph


PY2 = sys.version_info[0] == 2


def test_graph():

    # graph_0
    nodes = [{"name": "结点1", "symbolSize": 10},
             {"name": "结点2", "symbolSize": 20},
             {"name": "结点3", "symbolSize": 30},
             {"name": "结点4", "symbolSize": 40},
             {"name": "结点5", "symbolSize": 50},
             {"name": "结点6", "symbolSize": 40},
             {"name": "结点7", "symbolSize": 30},
             {"name": "结点8", "symbolSize": 20}]
    links = []
    for i in nodes:
        for j in nodes:
            links.append({"source": i.get('name'), "target": j.get('name')})
    graph = Graph("关系图-力引导布局示例")
    graph.add("", nodes, links, repulsion=8000)
    graph.show_config()
    graph.render()

    # graph_1
    graph = Graph("关系图-环形布局示例")
    graph.add("", nodes, links, is_label_show=True, repulsion=8000, layout='circular', label_text_color=None)
    graph.show_config()
    graph.render()

    # graph_2
    import json
    if PY2:
        import codecs
        with codecs.open(os.path.join("..", "json", "weibo.json"), "rb") as f:
            j = json.load(f)
    else:
        with open(os.path.join("..", "json", "weibo.json"), "r", encoding="utf-8") as f:
            j = json.load(f)
    nodes, links, categories, cont, mid, userl = j
    graph = Graph("微博转发关系图", width=1200, height=600)
    graph.add("", nodes, links, categories, label_pos="right", repulsion=50, is_legend_show=False, line_curve=0.2,
              label_text_color=None)
    graph.show_config()
    graph.render()



#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base
from pyecharts.option import get_all_options


class Funnel(Base):
    """
    <<< Funnel chart >>>
    """
    def __init__(self, title="", subtitle="", **kwargs):
        super(Funnel, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, attr, value, **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param attr:
            name of attribute
        :param value:
            value of attribute
        :param kwargs:
        """
        assert len(attr) == len(value)
        chart = get_all_options(**kwargs)
        _data = []
        for data in zip(attr, value):
            _name, _value = data
            _data.append({"name": _name, "value": _value})
        for a in attr:
            self._option.get('legend')[0].get('data').append(a)
        _dset = set(self._option.get('legend')[0].get('data'))
        self._option.get('legend')[0].update(data=list(_dset))
        self._option.get('series').append({
            "type": "funnel",
            "name": name,
            "data": _data,
            "label": chart['label']
        })
        self._legend_visualmap_colorlst(**kwargs)



#!/usr/bin/env python
#coding=utf-8

from pyecharts.base import Base
from pyecharts.option import get_all_options


class Bar(Base):
    """
    <<< Bar chart >>>
    Bar chart shows different data through the height of a bar,
    which is used in rectangular coordinate with at least 1 category axis.
    """

    def __init__(self, title="", subtitle="", **kwargs):
        super(Bar, self).__init__(title, subtitle, **kwargs)

    def add(self, *args, **kwargs):
        self.__add(*args, **kwargs)

    def __add(self, name, x_axis, y_axis,
              is_stack=False,
              **kwargs):
        """

        :param name:
            Series name used for displaying in tooltip and filtering with legend,
            or updaing data and configuration with setOption.
        :param x_axis:
            data of xAixs
        :param y_axis:
            data of yAxis
        :param is_stack:
            It specifies whether to stack category axis.
        :param kwargs:
        """
        assert len(x_axis) == len(y_axis)
        kwargs.update(x_axis=x_axis)
        chart = get_all_options(**kwargs)
        is_stack = "stack" if is_stack else ""
        xaxis, yaxis = chart['xy_axis']
        self._option.update(xAxis=xaxis, yAxis=yaxis)
        self._option.get('legend')[0].get('data').append(name)
        self._option.get('series').append({
            "type": "bar",
            "name": name,
            "data": y_axis,
            "stack": is_stack,
            "label": chart['label'],
            "markPoint": chart['mark_point'],
            "markLine": chart['mark_line'],
            "indexflag": self._option.get('_index_flag')
        })
        self._legend_visualmap_colorlst(**kwargs)
